@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -122,10 +122,12 @@ namespace H3D {
     /// \param thread_frequency The frequence of the thread loop. -1 means
     /// run as fast as possible.
     /// \param _useMainThread If true execute all callbacks in the main thread.
+    /// \param _use_synchronization If true, use deterministic synchronization of the physics and graphics threads.
     PhysicsEngineThread( const string &engine = "ODE",
                          Priority thread_priority = NORMAL_PRIORITY,
                          int thread_frequency = 100,
-                         bool _useMainThread = false );
+                         bool _useMainThread = false,
+                         bool _use_synchronization = false );
 
     /// Destructor.
     virtual ~PhysicsEngineThread();
@@ -211,7 +213,7 @@ namespace H3D {
     /// Remove a rigid body from the physics engine. Returns 0 on success.
     bool removeRigidBody( H3DBodyId id );
 
-    /// Set the parameters for a given rigid body. Only the paramaters in the
+    /// Set the parameters for a given rigid body. Only the parameters in the
     /// RigidBodyParameters structure that has actually been set will be
     /// changed.
     /// Returns 0 on success.
@@ -268,7 +270,7 @@ namespace H3D {
     /// Remove a collision space from the physics engine. Returns 0 on success.
     bool removeSpace( H3DSpaceId id );
 
-    /// TODO: implement
+    // Implement these functions.
     /*bool setSpaceParameters( H3DSpaceId space_id,
     SpaceParameters& p );
 
@@ -290,6 +292,9 @@ namespace H3D {
     /// Set the global world parameters of the physics engine.
     void setWorldParameters( WorldParameters& params );    
 
+    /// Synchronise with the scene graph
+    void synchroniseWithSceneGraph();
+
     /// Adds the contacts from the last collision detection phase to the given list.
     /// Returns 0 on success.
     int getCurrentContacts( list< ContactParameters > &contacts );
@@ -317,6 +322,22 @@ namespace H3D {
       return simulation_running;
     }
 
+    /// If the simulation is still executing steps requested by a previous call to
+    /// beginSimulationSteps() then this function will block, otherwise the function 
+    /// will return immediately.
+    void waitForSimulationSteps();
+
+    /// Begin executing a fixed number of simulation steps
+    ///
+    /// When called periodically from the graphics thread this allows for a deterministic
+    /// physics/graphics thread interaction while still allowing parallel execution.
+    ///
+    /// \param nr_steps The number of physics steps to execute
+    void beginSimulationSteps( std::size_t nr_steps );
+
+    /// Begin copying out the results of the previous physics simulation steps
+    void beginSynchronizeSteps();
+
     /// Set the global contact parameters for the engine, i.e. the parameters
     /// that will be used per default when two shapes collide.
     inline void setGlobalContactParameters( const GlobalContactParameters &params ) {
@@ -444,8 +465,9 @@ namespace H3D {
       H3DUtil::PeriodicThread::CallbackCode (*deInitEngine)(void *data);
       H3DUtil::PeriodicThread::CallbackCode (*doSimulationSteps)(void *data);
       H3DUtil::PeriodicThread::CallbackCode (*setWorldParameters)(void *data);
+      H3DUtil::PeriodicThread::CallbackCode (*synchroniseWithSceneGraph)(void *data);
       H3DUtil::PeriodicThread::CallbackCode (*getCurrentContacts)(void *data);
-
+      
       // shape callback functions
       H3DUtil::PeriodicThread::CallbackCode (*addCollidable)(void *data);
       H3DUtil::PeriodicThread::CallbackCode (*removeCollidable)(void *data);
@@ -509,6 +531,7 @@ namespace H3D {
       cb->doSimulationSteps = A::doSimulationSteps;
       cb->getCurrentContacts = A::getCurrentContacts;
       cb->setWorldParameters = A::setWorldParameters;
+      cb->synchroniseWithSceneGraph= A::synchroniseWithSceneGraph;
 
       // shape callback functions
       cb->addCollidable = A::addCollidable;
@@ -549,8 +572,31 @@ namespace H3D {
     /// to rigid bodies.
     void setHapticsDevices ( const NodeVector& _hapticsDevices );
 
+    /// Adds a node to be deleted in synch with the graphics thread.
+    /// One example use is to add a node which is not needed anymore but deleting it
+    /// in the physics thread would cause problems( due to routing etc. ) in the scene-graph.
+    /// Added nodes are deleted at the end of BodyCollection::traverseSG()
+    void addNodeToDeleteInSynch( Node *n );
+
+    /// Tells the physics engine thread to syncronise physics and graphics threads at the end
+    /// of the graphics frame.
+    void synchroniseAtTheEndOfTheFrame();
+
+    /// True if using deterministic synchronization of the physics and graphics threads
+    bool useSynchronization() {
+      return use_synchronization;
+    }
+
   protected:
 
+    /// Nodes populated from the physics thread but deleted in the graphics thread.
+    AutoRefVector < Node > nodesToDeleteInSync;
+    MutexLock nodesToDeleteInSync_lock;
+
+    /// If it is true synchronises the physics and graphics thread at the end of the graphics frame.
+    bool synchroniseTheThreads;
+    MutexLock synchroniseTheThreads_lock;
+
     PhysicsEngineParameters::ExternalForceTorqueParameters force_torque_params;
 
     NodeVector hapticsDevices;
@@ -583,17 +629,25 @@ namespace H3D {
       return ans;
     }
 
-    /// Callback function to update and save the update rate.
-    static H3DUtil::PeriodicThread::CallbackCode saveUpdateRate(void *data);
+    /// Callback function executed after stepping the simulation to finalize and copy out data
+    ///
+    /// Simply calls the method updateSimulation()
+    static H3DUtil::PeriodicThread::CallbackCode updateSimulationCB(void *data);
+
+    /// Callback function to execute a fixed number of physics frames
+    static H3DUtil::PeriodicThread::CallbackCode executeSimulationSteps( void* data );
 
     /// Adds external haptic interaction forces to a rigid body
     void addHapticInteractionForces ( 
       PhysicsEngineParameters::RigidBodyParameters& params, 
       PhysicsEngineParameters::ExternalForceTorqueParameters& forceParams );
 
+    /// A method executed after stepping the simulation to finalize and copy out data
+    virtual void updateSimulation();
+
     /// This function will be called once per cycle in the physics engine thread
     /// in order to update the output parameters of shapes, bodies and constraints.
-    void updateObjectParameters();
+    void updateRigidBodyParameters();
 
     /// Do pre-processing of the CollidableParameters before it is sent to the physics thread.
     ///
@@ -605,10 +659,17 @@ namespace H3D {
     ///
     void scaleGeometry ( X3DGeometryNode& geometry, const Vec3f& scale );
 
+    /// Helper function which returns true if the scale factor is uniform
+    static bool isUniformScale ( const Vec3f& scale );
+
+    /// Makes the scale parameter uniform and prints a warning message if it
+    /// is not already uniform
+    static Vec3f makeUniformScale ( const Vec3f& scale, X3DGeometryNode& geometry );
+
     /// The underlying physics engine that we currently use.
     string engine;
 
-    /// Internal data specific for the underying physics engine.
+    /// Internal data specific for the underlying physics engine.
     auto_ptr< EngineSpecificData > engine_data;
 
     /// True if the simulation is running.
@@ -618,8 +679,8 @@ namespace H3D {
     /// the simulation step.
     int simulation_cb_handle;
 
-    /// Thread callback function handle for saveUpdateRate callback function.
-    int save_update_cb_handle;
+    /// Thread callback function handle for updateSimulationCB callback function.
+    int update_cb_handle;
 
     /// Global contact parameters to be used by default on all contacts.
     GlobalContactParameters global_contact_params;
@@ -701,6 +762,20 @@ namespace H3D {
    typedef std::map<H3DBodyId,FILE*> DebugFileMap;
    DebugFileMap lagDebugFiles;
 #endif
+
+    /// A condition lock to synchronize physics and graphics threads in the
+    /// case that deterministic synchronization is used (see beginSimulationSteps())
+    ConditionLock stepping_simulation_lock;
+
+    /// The condition variable associated with stepping_simulation_lock. True if simulation
+    /// steps are currently executing.
+    bool stepping_simulation;
+
+    /// True if using deterministic synchronization of the physics and graphics threads
+    bool use_synchronization;
+
+    /// The number of simulation steps requested by beginSimulationSteps()
+    std::size_t nr_physics_steps;
   };
 
 
