@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -37,11 +37,14 @@ using namespace H3D;
 
 BulletJoint::JointDatabase::JointMap BulletJoint::JointDatabase::map;
 
+const H3DFloat BulletJoint::maxMotorForce = 1e5;
+const H3DFloat BulletJoint::maxMotorTorque = 1e5;
+
 BulletJoint::BulletJoint ( PhysicsEngineParameters::JointParameters& jointParameters ) :
 physicsEngineThread ( jointParameters.getEngine() ),
-joint ( NULL ),
-body1 ( NULL ),
-body2 ( NULL )
+  joint ( NULL ),
+  body1 ( NULL ),
+  body2 ( NULL )
 {
   // Get bullet representations of rigid bod(y/ies)
   if ( BulletBody* b= (BulletBody*)jointParameters.getBody1() ) {
@@ -76,7 +79,7 @@ void BulletJoint::add ()
   BulletCallbacks::BulletSpecificData* bullet_data= 
     static_cast<BulletCallbacks::BulletSpecificData*>(physicsEngineThread->getEngineSpecificData());
 
-  bullet_data->m_dynamicsWorld->addConstraint( joint.get() );
+  bullet_data->m_dynamicsWorld->addConstraint( joint.get(), true);
 }
 
 // Remove joint from simulation
@@ -96,11 +99,11 @@ btRigidBody& BulletJoint::getFixedBody()
 }
 
 BulletBallJoint::JointDatabase
-BulletBallJoint::database ( "BallJoint", &newInstance<BulletBallJoint> );
+  BulletBallJoint::database ( "BallJoint", &newInstance<BulletBallJoint> );
 
 BulletBallJoint::BulletBallJoint ( PhysicsEngineParameters::JointParameters& jointParameters ) :
 BulletJoint ( jointParameters ),
-ballJoint ( NULL )
+  ballJoint ( NULL )
 {
 
   PhysicsEngineParameters::BallJointParameters* ballJointParameters=
@@ -175,13 +178,15 @@ void BulletBallJoint::getParameters ( PhysicsEngineParameters::JointParameters&
 }
 
 BulletSingleAxisHingeJoint::JointDatabase
-BulletSingleAxisHingeJoint::database ( "SingleAxisHingeJoint", &newInstance<BulletSingleAxisHingeJoint> );
+  BulletSingleAxisHingeJoint::database ( "SingleAxisHingeJoint", &newInstance<BulletSingleAxisHingeJoint> );
 
 BulletSingleAxisHingeJoint::BulletSingleAxisHingeJoint ( PhysicsEngineParameters::JointParameters& jointParameters ) :
 BulletJoint ( jointParameters ),
-hingeJoint ( NULL ),
-anchorPoint ( btVector3 ( 0, 0, 0 ) ),
-bounce ( 0 )
+  hingeJoint ( NULL ),
+  anchorPoint ( btVector3 ( 0.f, 0.f, 0.f ) ),
+  bounce ( 0.f ),
+  bias( 0.3f ),
+  softness( 0.4f ) 
 {
   PhysicsEngineParameters::SingleAxisHingeJointParameters* hingeJointParameters= 
     dynamic_cast < PhysicsEngineParameters::SingleAxisHingeJointParameters* > ( &jointParameters );
@@ -213,7 +218,6 @@ bounce ( 0 )
   hingeJoint= new btHingeConstraint ( *body1, *body2, 
     localAnchorPoint1, localAnchorPoint2,
     localAxis1, localAxis2 );
-
   joint.reset ( hingeJoint );
 }
 
@@ -225,6 +229,11 @@ void BulletSingleAxisHingeJoint::setParameters ( PhysicsEngineParameters::JointP
   BulletCallbacks::BulletSpecificData* bullet_data= 
     static_cast<BulletCallbacks::BulletSpecificData*>(hingeJointParameters->getEngine()->getEngineSpecificData());
 
+  // Motor velocity
+  if( hingeJointParameters->haveMotorTarget() ) {
+    hingeJoint->enableAngularMotor( true, hingeJointParameters->getMotorTarget(), maxMotorTorque );
+  }
+
   // Set hinge anchor point
   if( hingeJointParameters->haveAnchorPoint() ) 
   {
@@ -244,7 +253,7 @@ void BulletSingleAxisHingeJoint::setParameters ( PhysicsEngineParameters::JointP
     maxAngle= hingeJointParameters->getMaxAngle();
   }
   if ( hingeJointParameters->haveMinAngle() || hingeJointParameters->haveMaxAngle() ) {
-    hingeJoint->setLimit( minAngle,maxAngle, btScalar(0.0), btScalar(0.3), bounce );
+    hingeJoint->setLimit( minAngle,maxAngle, softness, bias, bounce );
   }
 
   // Set bounce value for joint limits (note negation required to convert to CFM)
@@ -254,9 +263,19 @@ void BulletSingleAxisHingeJoint::setParameters ( PhysicsEngineParameters::JointP
     // _biasFactor works as a multiplier to constraint error, constraint becomes more "soft" when this factor is close to zero;
     // _relaxationFactor used to control bounce on limits : 1.0 means full bounce, 0.0 means no bounce
     bounce= btScalar(hingeJointParameters->getStopBounce());
-    hingeJoint->setLimit( hingeJoint->getLowerLimit(),
-      hingeJoint->getUpperLimit(), btScalar(0.0), btScalar(0.3), 
-      bounce );
+    hingeJoint->setLimit( hingeJoint->getLowerLimit(), hingeJoint->getUpperLimit(), softness, bias, bounce );
+  }
+
+  if( hingeJointParameters->haveBias() )
+  {
+    bias = btScalar(hingeJointParameters->getBias());
+    hingeJoint->setLimit( hingeJoint->getLowerLimit(), hingeJoint->getUpperLimit(), softness, bias, bounce);
+  }
+
+  if( hingeJointParameters->haveSoftness() )
+  {
+    softness = btScalar(hingeJointParameters->getSoftness());
+    hingeJoint->setLimit( hingeJoint->getLowerLimit(), hingeJoint->getUpperLimit(), softness, bias, bounce);
   }
 
   // Set error correction value for joint limits
@@ -280,7 +299,7 @@ void BulletSingleAxisHingeJoint::setParameters ( PhysicsEngineParameters::JointP
 
     btTransform frameB= body2->getWorldTransform().inverse() * constraintSpace;
     hingeJoint->getBFrame()= frameB;
-  }
+}
 }
 
 // Modify the jointParameters argument to reflect the bullet joint's current parameters
@@ -300,16 +319,38 @@ void BulletSingleAxisHingeJoint::getParameters ( PhysicsEngineParameters::JointP
   if ( hingeJointParameters->haveBody2AnchorPoint() ) {
     if ( body1 && body2 ) {
       hingeJointParameters->setBody2AnchorPoint ( toVec3f ( body2->getWorldTransform()*hingeJoint->getBFrame().getOrigin()
-      / bullet_data->m_worldScale ) );
+        / bullet_data->m_worldScale ) );
     } else {
       hingeJointParameters->setBody2AnchorPoint ( toVec3f ( body2->getWorldTransform()*hingeJoint->getAFrame().getOrigin()
-      / bullet_data->m_worldScale ) );
+        / bullet_data->m_worldScale ) );
     }
   }
 
+  //********************************************************************************//
+  btVector3 localY = btVector3(1,0,0);
+  btVector3 localX = btVector3(0,1,0);
+
+  btVector3 bodyA_Y = hingeJoint->getRigidBodyB().getWorldTransform().getBasis().inverse() * localY;
+  btVector3 bodyB_Y = hingeJoint->getRigidBodyA().getWorldTransform().getBasis().inverse() * localY;
+  btVector3 bodyB_X = hingeJoint->getRigidBodyA().getWorldTransform().getBasis().inverse() * localX;
+
+  // Project bodyA_Y on to plane with normal bodyB_X
+  btVector3 projBodyA_Y= bodyA_Y - bodyA_Y.dot( bodyB_X ) * bodyB_X;
+  projBodyA_Y.normalize();
+  
+  // Get the angle between the two Y axes now in the same plane
+  // Using the fact that for unit vectors the following is true atan2( sin(angle), cos(angle) ) = atan2( ||a.cross(b)||, a.dot(b) )
+  btVector3 bodyB_Y_norm = bodyB_Y; 
+  bodyB_Y_norm.normalize();
+  btScalar angle= H3DAtan2( ( projBodyA_Y.cross( bodyB_Y_norm ) ).length(), projBodyA_Y.dot( bodyB_Y_norm ) );
+
+  //********************************************************************************//
+  
   // Get hinge angle
   if( hingeJointParameters->haveAngle() )
+  {
     hingeJointParameters->setAngle( hingeJoint->getHingeAngle() );
+  }
 
   // angleRate
   if ( hingeJointParameters->haveAngleRate() )
@@ -320,14 +361,14 @@ void BulletSingleAxisHingeJoint::getParameters ( PhysicsEngineParameters::JointP
 }
 
 BulletUniversalJoint::JointDatabase
-BulletUniversalJoint::database ( "UniversalJoint", &newInstance<BulletUniversalJoint> );
+  BulletUniversalJoint::database ( "UniversalJoint", &newInstance<BulletUniversalJoint> );
 
 BulletUniversalJoint::BulletUniversalJoint ( PhysicsEngineParameters::JointParameters& jointParameters ) :
 BulletJoint ( jointParameters ),
-universalJoint ( NULL ),
-anchorPoint ( btVector3 ( 0, 0, 0 ) ),
-axis1 ( btVector3 ( 1, 0, 0 ) ),
-axis2 ( btVector3 ( 0, 1, 0 ) )
+  universalJoint ( NULL ),
+  anchorPoint ( btVector3 ( 0, 0, 0 ) ),
+  axis1 ( btVector3 ( 1, 0, 0 ) ),
+  axis2 ( btVector3 ( 0, 1, 0 ) )
 {
   PhysicsEngineParameters::UniversalJointParameters* universalJointParameters=
     dynamic_cast<PhysicsEngineParameters::UniversalJointParameters*> ( &jointParameters );
@@ -450,16 +491,16 @@ void BulletUniversalJoint::updateFrames ()
 }
 
 BulletDoubleAxisHingeJoint::JointDatabase
-BulletDoubleAxisHingeJoint::database ( "DoubleAxisHingeJoint", &newInstance<BulletDoubleAxisHingeJoint> );
+  BulletDoubleAxisHingeJoint::database ( "DoubleAxisHingeJoint", &newInstance<BulletDoubleAxisHingeJoint> );
 
 BulletDoubleAxisHingeJoint::BulletDoubleAxisHingeJoint ( PhysicsEngineParameters::JointParameters& jointParameters ) :
 BulletJoint ( jointParameters ),
-doubleHingeJoint ( NULL ),
-anchorPoint ( btVector3 ( 0, 0, 0 ) ),
-axis1 ( btVector3 ( 1, 0, 0 ) ),
-axis2 ( btVector3 ( 0, 1, 0 ) ),
-suspensionERP ( 0 ),
-suspensionCFM ( 0 )
+  doubleHingeJoint ( NULL ),
+  anchorPoint ( btVector3 ( 0, 0, 0 ) ),
+  axis1 ( btVector3 ( 1, 0, 0 ) ),
+  axis2 ( btVector3 ( 0, 1, 0 ) ),
+  suspensionERP ( 0 ),
+  suspensionCFM ( 0 )
 {
   PhysicsEngineParameters::DoubleAxisHingeJointParameters* doubleHingeJointParameters=
     dynamic_cast<PhysicsEngineParameters::DoubleAxisHingeJointParameters*> ( &jointParameters );
@@ -565,9 +606,19 @@ void BulletDoubleAxisHingeJoint::setParameters ( PhysicsEngineParameters::JointP
     doubleHingeJointParameters->haveSuspensionErrorCorrection() )
     updateSuspension ( *doubleHingeJointParameters );
 
+#if BT_BULLET_VERSION < 283
   // stopBounce1 - restitution
   if ( doubleHingeJointParameters->haveStopBounce1() )
     doubleHingeJoint->getRotationalLimitMotor(2)->m_limitSoftness= btScalar ( doubleHingeJointParameters->getStopBounce1() );
+#else
+#ifdef _MSC_VER
+#pragma message("Bullet version is equal to or above 2.83. stopBounce1 field of DoubleAxisHingeJoint will not be used.") 
+#else
+#ifdef __GNUC__
+#warning Bullet version is equal to or above 2.83. stopBounce1 field of DoubleAxisHingeJoint will not be used.
+#endif
+#endif
+#endif
 
   // stopConstantForceMix1
   if ( doubleHingeJointParameters->haveStopConstantForceMix1() )
@@ -682,27 +733,27 @@ void BulletDoubleAxisHingeJoint::updateSuspension ( PhysicsEngineParameters::Dou
 }
 
 BulletGeneric6DOFJoint::JointDatabase
-BulletGeneric6DOFJoint::database ( "Generic6DOFJoint", &newInstance<BulletGeneric6DOFJoint> );
+  BulletGeneric6DOFJoint::database ( "Generic6DOFJoint", &newInstance<BulletGeneric6DOFJoint> );
 
 BulletGeneric6DOFJoint::BulletGeneric6DOFJoint ( PhysicsEngineParameters::JointParameters& jointParameters ) :
 BulletJoint ( jointParameters ),
-generic6DOFJoint ( NULL ),
-anchorPoint ( btVector3 ( 0, 0, 0 ) ),
-axis1 ( btVector3 ( 1, 0, 0 ) ),
-axis2 ( btVector3 ( 0, 1, 0 ) ),
-axis3 ( btVector3 ( 0, 0, 1 ) ),
-maxAngle1 ( 0.0f ),
-maxAngle2 ( 0.0f ),
-maxAngle3 ( 0.0f ),
-minAngle1 ( 0.0f ),
-minAngle2 ( 0.0f ),
-minAngle3 ( 0.0f ),
-maxLimit1 ( 0.0f ),
-maxLimit2 ( 0.0f ),
-maxLimit3 ( 0.0f ),
-minLimit1 ( 0.0f ),
-minLimit2 ( 0.0f ),
-minLimit3 ( 0.0f )
+  generic6DOFJoint ( NULL ),
+  anchorPoint ( btVector3 ( 0, 0, 0 ) ),
+  axis1 ( btVector3 ( 1, 0, 0 ) ),
+  axis2 ( btVector3 ( 0, 1, 0 ) ),
+  axis3 ( btVector3 ( 0, 0, 1 ) ),
+  maxAngle1 ( 0.0f ),
+  maxAngle2 ( 0.0f ),
+  maxAngle3 ( 0.0f ),
+  minAngle1 ( 0.0f ),
+  minAngle2 ( 0.0f ),
+  minAngle3 ( 0.0f ),
+  maxLimit1 ( 0.0f ),
+  maxLimit2 ( 0.0f ),
+  maxLimit3 ( 0.0f ),
+  minLimit1 ( 0.0f ),
+  minLimit2 ( 0.0f ),
+  minLimit3 ( 0.0f )
 {
   PhysicsEngineParameters::Generic6DOFJointParameters* generic6DOFJointParameters=
     dynamic_cast<PhysicsEngineParameters::Generic6DOFJointParameters*> ( &jointParameters );
@@ -920,15 +971,15 @@ void BulletGeneric6DOFJoint::updateFrames ()
 }
 
 BulletSliderJoint::JointDatabase
-BulletSliderJoint::database ( "SliderJoint", &newInstance<BulletSliderJoint> );
+  BulletSliderJoint::database ( "SliderJoint", &newInstance<BulletSliderJoint> );
 
 BulletSliderJoint::BulletSliderJoint ( PhysicsEngineParameters::JointParameters& jointParameters ) :
 BulletJoint ( jointParameters ),
-sliderJoint ( NULL ),
-minSeparation ( 0 ),
-maxSeparation ( 0 ),
-slider_force( 0 ),
-simulationCallbackId( -1 )
+  sliderJoint ( NULL ),
+  minSeparation ( 0 ),
+  maxSeparation ( 0 ),
+  slider_force( 0 ),
+  simulationCallbackId( -1 )
 {
   PhysicsEngineParameters::SliderJointParameters* sliderJointParameters=
     dynamic_cast<PhysicsEngineParameters::SliderJointParameters*> ( &jointParameters );
@@ -945,11 +996,16 @@ simulationCallbackId( -1 )
   setAxis ( sliderJointParameters->getAxis(), frameA, frameB );
 
   // Create slider joint (frame a and b are set later in setParameters)
-  sliderJoint= new btGeneric6DofConstraint ( *body1, *body2, frameA, frameB, false );
-  sliderJoint->setAngularLowerLimit ( btVector3 ( 0, 0, 0 ) );
-  sliderJoint->setAngularUpperLimit ( btVector3 ( 0, 0, 0 ) );
-  sliderJoint->setLinearUpperLimit ( btVector3 ( 0, 0, 0 ) );
-  sliderJoint->setLinearLowerLimit ( btVector3 ( 0, 0, 0 ) );
+
+  //sliderJoint= new btGeneric6DofConstraint ( *body1, *body2, frameA, frameB, false );
+  //sliderJoint->setAngularLowerLimit ( btVector3 ( 0, 0, 0 ) );
+  //sliderJoint->setAngularUpperLimit ( btVector3 ( 0, 0, 0 ) );
+  //sliderJoint->setLinearUpperLimit ( btVector3 ( 0, 0, 0 ) );
+  //sliderJoint->setLinearLowerLimit ( btVector3 ( 0, 0, 0 ) );
+  ////sliderJoint->getRigidBodyA().setAngularFactor(btVector3(1,0,0));
+  ////sliderJoint->getRigidBodyB().setAngularFactor(btVector3(1,0,0));
+
+  sliderJoint= new btSliderConstraint ( *body1, *body2, frameA, frameB, true );
 
   joint.reset ( sliderJoint );
 }
@@ -962,6 +1018,13 @@ void BulletSliderJoint::setParameters ( PhysicsEngineParameters::JointParameters
   BulletCallbacks::BulletSpecificData* bullet_data= 
     static_cast<BulletCallbacks::BulletSpecificData*>(sliderJointParameters->getEngine()->getEngineSpecificData());
 
+  // Motor velocity
+  if( sliderJointParameters->haveMotorTarget() ) {
+    sliderJoint->setPoweredLinMotor( true );
+    sliderJoint->setMaxLinMotorForce( maxMotorForce );
+    sliderJoint->setTargetLinMotorVelocity ( -sliderJointParameters->getMotorTarget() );
+  }
+
   // axis
   if ( sliderJointParameters->haveAxis() )
     setAxis ( sliderJointParameters->getAxis(), sliderJoint->getFrameOffsetA(), sliderJoint->getFrameOffsetB() );
@@ -969,20 +1032,25 @@ void BulletSliderJoint::setParameters ( PhysicsEngineParameters::JointParameters
   // minSeparation
   if( sliderJointParameters->haveMinSeparation() )
     minSeparation= sliderJointParameters->getMinSeparation()*bullet_data->m_worldScale;
-    
+
 
   // maxSepartion
   if( sliderJointParameters->haveMaxSeparation() )
     maxSeparation= sliderJointParameters->getMaxSeparation()*bullet_data->m_worldScale;
 
   if ( sliderJointParameters->haveMinSeparation() || sliderJointParameters->haveMaxSeparation() ) {
-    sliderJoint->setLinearUpperLimit ( btVector3 ( -minSeparation, 0, 0 ) );
-    sliderJoint->setLinearLowerLimit ( btVector3 ( -maxSeparation, 0, 0 ) );
+    //sliderJoint->setLinearUpperLimit ( btVector3 ( -minSeparation, 0, 0 ) );
+    //sliderJoint->setLinearLowerLimit ( btVector3 ( -maxSeparation, 0, 0 ) );
+    sliderJoint->setUpperLinLimit (-minSeparation );
+    sliderJoint->setLowerLinLimit (-maxSeparation );
+    //sliderJoint->setPoweredLinMotor(true);
+
   }
 
   // Set bounce value for joint limits, a.k.a restitution
   if( sliderJointParameters->haveStopBounce() )
-    sliderJoint->getTranslationalLimitMotor()->m_restitution= sliderJointParameters->getStopBounce();
+    //sliderJoint->getTranslationalLimitMotor()->m_restitution= sliderJointParameters->getStopBounce();
+    sliderJoint->setRestitutionLimLin(btScalar(sliderJointParameters->getStopBounce()));
 
   // Set error correction value for joint limits
   if( sliderJointParameters->haveStopErrorCorrection() )
@@ -996,7 +1064,7 @@ void BulletSliderJoint::setParameters ( PhysicsEngineParameters::JointParameters
     } else if( slider_force == 0 ) {
       physicsEngineThread->removeAsynchronousCallbackNoLock( simulationCallbackId );
       simulationCallbackId = -1;
-    }
+}
   } else if( simulationCallbackId >= 0 ) {
     physicsEngineThread->removeAsynchronousCallbackNoLock( simulationCallbackId );
     simulationCallbackId = -1;
@@ -1011,14 +1079,20 @@ void BulletSliderJoint::getParameters ( PhysicsEngineParameters::JointParameters
   BulletCallbacks::BulletSpecificData* bullet_data= 
     static_cast<BulletCallbacks::BulletSpecificData*>(sliderJointParameters->getEngine()->getEngineSpecificData());
 
-  if ( sliderJointParameters->haveSeparation() ) {
-    sliderJointParameters->setSeparation ( -sliderJoint->getRelativePivotPosition(0) / bullet_data->m_worldScale );
+  btVector3 diff = sliderJoint->getRigidBodyB().getCenterOfMassPosition() - sliderJoint->getRigidBodyA().getCenterOfMassPosition();
+  H3DFloat sep = diff.length();
+  btVector3 globalAxis = body1->getWorldTransform().inverse() * btVector3(1,0,0);
+  if ( diff.dot(globalAxis) < 0) {
+    sep = -sep;
+  }
+  if( sliderJointParameters->haveSeparation() ) {
+    sliderJointParameters->setSeparation ( -sliderJoint->getLinearPos() / bullet_data->m_worldScale );
   }
 
   // SeparationRate rate of change of slider separation
-  if ( sliderJointParameters->haveSeparationRate() ) {
-   separationRate.setCurrentValue ( -sliderJoint->getRelativePivotPosition(0) / bullet_data->m_worldScale );
-    sliderJointParameters->setSeparationRate ( separationRate );
+  if( sliderJointParameters->haveSeparationRate() ) {
+    separationRate.setCurrentValue ( -sliderJoint->getLinearPos() / bullet_data->m_worldScale );
+    sliderJointParameters->setSeparationRate( separationRate );
   }
 }
 
@@ -1086,18 +1160,20 @@ void BulletSliderJoint::remove ()
     physicsEngineThread->removeAsynchronousCallbackNoLock ( simulationCallbackId );
     simulationCallbackId = -1;
   }
+
+  BulletJoint::remove();
 }
 
 BulletMotorJoint::JointDatabase
-BulletMotorJoint::database ( "MotorJoint", &newInstance<BulletMotorJoint> );
+  BulletMotorJoint::database ( "MotorJoint", &newInstance<BulletMotorJoint> );
 
 BulletMotorJoint::BulletMotorJoint ( PhysicsEngineParameters::JointParameters& jointParameters ) :
 BulletJoint ( jointParameters ),
-simulationCallbackId ( -1 ),
-torque1 ( 0 ),
-torque2 ( 0 ),
-torque3 ( 0 ),
-enabledAxes ( 0 )
+  simulationCallbackId ( -1 ),
+  torque1 ( 0 ),
+  torque2 ( 0 ),
+  torque3 ( 0 ),
+  enabledAxes ( 0 )
 {
 }
 
@@ -1230,12 +1306,12 @@ H3DUtil::PeriodicThread::CallbackCode BulletMotorJoint::updateMotor ( void *data
 }
 
 BulletSoftBodyLinearJoint::JointDatabase
-BulletSoftBodyLinearJoint::database ( "SoftBodyLinearJoint", &newInstance<BulletSoftBodyLinearJoint> );
+  BulletSoftBodyLinearJoint::database ( "SoftBodyLinearJoint", &newInstance<BulletSoftBodyLinearJoint> );
 
 // A class to manage the link between a SoftBodyLinearJoint and its Bullet implementation
 BulletSoftBodyLinearJoint::BulletSoftBodyLinearJoint ( JointParameters& jointParameters ) 
-: BulletJoint ( jointParameters ),
-softBody1 ( NULL ), softBody2 ( NULL ), index ( -1 )
+  : BulletJoint ( jointParameters ),
+  softBody1 ( NULL ), softBody2 ( NULL ), bt_joint ( NULL )
 {
   SoftBodyLinearJointParameters* sbLinearJointParameters=
     dynamic_cast<SoftBodyLinearJointParameters*> ( &jointParameters );
@@ -1267,7 +1343,7 @@ void BulletSoftBodyLinearJoint::setParameters ( JointParameters& jointParameters
 
   if ( sbLinearJointParameters->haveAnchorPoint() ) {
     anchorPoint= sbLinearJointParameters->getAnchorPoint() * bullet_data->m_worldScale;
-    if ( index >= 0 ) {
+    if ( bt_joint != NULL ) {
       // If currently added, re-add to refresh anchor point
       remove();
       add();
@@ -1285,7 +1361,7 @@ void BulletSoftBodyLinearJoint::add () {
   specs.position= tobtVector3 ( anchorPoint );
 
   // Save the joint index for removal
-  index= softBody1->m_joints.size();
+  int nr_joints_before = softBody1->m_joints.size();
 
   // Create and add the joint
   if ( softBody2 ) {
@@ -1298,30 +1374,31 @@ void BulletSoftBodyLinearJoint::add () {
     // Second body omitted: Soft body to static rigid body
     softBody1->appendLinearJoint ( specs, btSoftBody::Body ( &getFixedBody() ) );
   }
+
+  if( softBody1->m_joints.size() > nr_joints_before ) {
+    bt_joint = softBody1->m_joints[nr_joints_before];
+  }
 }
 
 void BulletSoftBodyLinearJoint::remove () {
-  /// \todo Regulary check if bullet have implemented a removeJoint feature.
-  /// Since the code below will cause a memory leak because bullet can not release data
-  /// allocated in appendLinearJoint and we can not do it either because it was allocated
-  /// in a different library (or could be if we are not linking statically).
-  if ( softBody1 && index >= 0 ) {
-    for ( int i= index; 
-      i+1 < softBody1->m_joints.size(); ++i ) {
-        softBody1->m_joints[i]= softBody1->m_joints[i+1];
+  if( bt_joint ) {
+    if( softBody1->m_joints.findLinearSearch( bt_joint ) != softBody1->m_joints.size() ) {
+      // This will set the joint up for deletion if the soft body simulation is still running
+      // If the simulation is not running anymore then the joint will be cleaned up
+      // when the soft body is destroyed.
+      bt_joint->m_delete = true;
+      bt_joint = NULL;
     }
-    softBody1->m_joints.resize ( softBody1->m_joints.size()-1 );
-    index= -1;
   }
 }
 
 BulletSoftBodyAngularJoint::JointDatabase
-BulletSoftBodyAngularJoint::database ( "SoftBodyAngularJoint", &newInstance<BulletSoftBodyAngularJoint> );
+  BulletSoftBodyAngularJoint::database ( "SoftBodyAngularJoint", &newInstance<BulletSoftBodyAngularJoint> );
 
 // A class to manage the link between a BulletSoftBodyAngularJoint and its Bullet implementation
 BulletSoftBodyAngularJoint::BulletSoftBodyAngularJoint ( JointParameters& jointParameters ) 
-: BulletJoint ( jointParameters ),
-softBody1 ( NULL ), softBody2 ( NULL ), index ( -1 )
+  : BulletJoint ( jointParameters ),
+  softBody1 ( NULL ), softBody2 ( NULL ), bt_joint ( NULL )
 {
   SoftBodyAngularJointParameters* sbAngularJointParameters=
     dynamic_cast<SoftBodyAngularJointParameters*> ( &jointParameters );
@@ -1349,7 +1426,7 @@ void BulletSoftBodyAngularJoint::setParameters ( JointParameters& jointParameter
 
   if ( sbAngularJointParameters->haveAxis() ) {
     axis= sbAngularJointParameters->getAxis();
-    if ( index >= 0 ) {
+    if ( bt_joint != NULL ) {
       // If currently added, re-add to refresh anchor point
       remove();
       add();
@@ -1367,7 +1444,7 @@ void BulletSoftBodyAngularJoint::add () {
   specs.axis= tobtVector3 ( axis );
 
   // Save the joint index for removal
-  index= softBody1->m_joints.size();
+  int nr_joints_before = softBody1->m_joints.size();
 
   // Create and add the joint
   if ( softBody2 ) {
@@ -1380,20 +1457,21 @@ void BulletSoftBodyAngularJoint::add () {
     // Second body omitted: Soft body to static rigid body
     softBody1->appendAngularJoint ( specs, btSoftBody::Body ( &getFixedBody() ) );
   }
+
+  if( softBody1->m_joints.size() > nr_joints_before ) {
+    bt_joint = softBody1->m_joints[nr_joints_before];
+  }
 }
 
 void BulletSoftBodyAngularJoint::remove () {
-  /// \todo Regulary check if bullet have implemented a removeJoint feature.
-  /// Since the code below will cause a memory leak because bullet can not release data
-  /// allocated in appendAngularJoint and we can not do it either because it was allocated
-  /// in a different library (or could be if we are not linking statically).
-  if ( softBody1 && index >= 0 ) {
-    for ( int i= index; 
-      i+1 < softBody1->m_joints.size(); ++i ) {
-        softBody1->m_joints[i]= softBody1->m_joints[i+1];
+  if( bt_joint ) {
+    if( softBody1->m_joints.findLinearSearch( bt_joint ) != softBody1->m_joints.size() ) {
+      // This will set the joint up for deletion if the soft body simulation is still running
+      // If the simulation is not running anymore then the joint will be cleaned up
+      // when the soft body is destroyed.
+      bt_joint->m_delete = true;
+      bt_joint = NULL;
     }
-    softBody1->m_joints.resize ( softBody1->m_joints.size()-1 );
-    index= -1;
   }
 }
 
