@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -38,7 +38,7 @@ H3DNodeDatabase IndexedElementSet::database( "IndexedElementSet",
                                             &X3DComposedGeometryNode::database);
 
 namespace IndexedElementSetInternals {
-  FIELDDB_ELEMENT( IndexedElementSet, index, INPUT_OUTPUT );
+  FIELDDB_ELEMENT( IndexedElementSet, index, INPUT_OUTPUT )
 }
 
 IndexedElementSet::IndexedElementSet (Inst< SFNode           > _metadata,
@@ -266,52 +266,52 @@ void IndexedElementSet::renderHexa ( unsigned int a, unsigned int b, unsigned in
   renderVertex ( d );
 }
 
-void IndexedElementSet::renderVertex ( unsigned int index )
+void IndexedElementSet::renderVertex ( unsigned int _index )
 {
-  if( color_node ) color_node->render( index );
-  if( tex_coords_per_vertex ) renderTexCoord( index,
+  if( color_node ) color_node->render( _index );
+  if( tex_coords_per_vertex ) renderTexCoord( _index,
     tex_coord_node );
-  if( fog_coord_node) fog_coord_node->render(index);
+  if( fog_coord_node) fog_coord_node->render(_index);
   for( unsigned int attrib_index = 0;
     attrib_index < attrib->size(); ++attrib_index ) {
       X3DVertexAttributeNode *attr = 
         attrib->getValueByIndex( attrib_index );
-      if( attr ) attr->render( index );
+      if( attr ) attr->render( _index );
   }
-  coordinate_node->render( index );
+  coordinate_node->render( _index );
 }
 
 void IndexedElementSet::AutoNormal::update() {
   bool normals_per_vertex = 
     static_cast< SFBool * >( routes_in[0] )->getValue();
-  X3DCoordinateNode *coord = 
+  X3DCoordinateNode *_coord = 
     static_cast< X3DCoordinateNode * >( static_cast< SFCoordinateNode * >
     ( routes_in[1] )->getValue() );
-  const vector<int> &index = 
+  const vector<int> &_index = 
     static_cast< MFInt32 * >( routes_in[2] )->getValue();
-  bool ccw = static_cast< SFBool * >( routes_in[3] )->getValue();
+  bool _ccw = static_cast< SFBool * >( routes_in[3] )->getValue();
 
   if( normals_per_vertex ) 
-    value = generateNormalsPerVertex( coord, index, ccw );
+    value = generateNormalsPerVertex( _coord, _index, _ccw );
   else
-    value = generateNormalsPerFace( coord, index, ccw );
+    value = generateNormalsPerFace( _coord, _index, _ccw );
 }
 
 X3DNormalNode *IndexedElementSet::AutoNormal::generateNormalsPerVertex( 
-  X3DCoordinateNode *coord,
-  const vector< int > &index,
-  bool ccw ) {
-    Normal *normal = new Normal;
-    if( coord ) {
-      vector< Vec3f > normals( coord->nrAvailableCoords(), 
+  X3DCoordinateNode *_coord,
+  const vector< int > &_index,
+  bool _ccw ) {
+    Normal *_normal = new Normal;
+    if( _coord ) {
+      vector< Vec3f > normals( _coord->nrAvailableCoords(), 
         Vec3f( 0, 0, 0 ) );
 
-      for( unsigned int i = 0; i < index.size(); ++i ) {
+      for( unsigned int i = 0; i < _index.size(); ++i ) {
 
         std::vector<int> elementIndices;
 
-        for(;  i < index.size() && index[i] != -1; ++i )
-          elementIndices.push_back( index[i] );
+        for(;  i < _index.size() && _index[i] != -1; ++i )
+          elementIndices.push_back( _index[i] );
 
         if( elementIndices.size() == 1 )
         {
@@ -323,22 +323,22 @@ X3DNormalNode *IndexedElementSet::AutoNormal::generateNormalsPerVertex(
 
           Vec3f norm;
 
-          norm= triangleNormal ( elementIndices[0], elementIndices[1], elementIndices[2], coord, ccw );
+          norm= triangleNormal ( elementIndices[0], elementIndices[1], elementIndices[2], _coord, _ccw );
           normals[elementIndices[0]] += norm;
           normals[elementIndices[1]] += norm;
           normals[elementIndices[2]] += norm;
 
-          norm= triangleNormal ( elementIndices[0], elementIndices[2], elementIndices[3], coord, ccw );
+          norm= triangleNormal ( elementIndices[0], elementIndices[2], elementIndices[3], _coord, _ccw );
           normals[elementIndices[0]] += norm;
           normals[elementIndices[2]] += norm;
           normals[elementIndices[3]] += norm;
 
-          norm= triangleNormal ( elementIndices[0], elementIndices[3], elementIndices[1], coord, ccw );
+          norm= triangleNormal ( elementIndices[0], elementIndices[3], elementIndices[1], _coord, _ccw );
           normals[elementIndices[0]] += norm;
           normals[elementIndices[3]] += norm;
           normals[elementIndices[1]] += norm;
 
-          norm= triangleNormal ( elementIndices[1], elementIndices[3], elementIndices[2], coord, ccw );
+          norm= triangleNormal ( elementIndices[1], elementIndices[3], elementIndices[2], _coord, _ccw );
           normals[elementIndices[1]] += norm;
           normals[elementIndices[3]] += norm;
           normals[elementIndices[2]] += norm;
@@ -347,37 +347,37 @@ X3DNormalNode *IndexedElementSet::AutoNormal::generateNormalsPerVertex(
 
           Vec3f norm;
 
-          norm= hexaNormal ( elementIndices[0], elementIndices[1], elementIndices[2], elementIndices[3], coord, ccw );
+          norm= hexaNormal ( elementIndices[0], elementIndices[1], elementIndices[2], elementIndices[3], _coord, _ccw );
           normals[elementIndices[0]] += norm;
           normals[elementIndices[1]] += norm;
           normals[elementIndices[2]] += norm;
           normals[elementIndices[3]] += norm;
 
-          norm= hexaNormal ( elementIndices[0], elementIndices[4], elementIndices[5], elementIndices[1], coord, ccw );
+          norm= hexaNormal ( elementIndices[0], elementIndices[4], elementIndices[5], elementIndices[1], _coord, _ccw );
           normals[elementIndices[0]] += norm;
           normals[elementIndices[4]] += norm;
           normals[elementIndices[5]] += norm;
           normals[elementIndices[1]] += norm;
 
-          norm= hexaNormal ( elementIndices[1], elementIndices[5], elementIndices[6], elementIndices[2], coord, ccw );
+          norm= hexaNormal ( elementIndices[1], elementIndices[5], elementIndices[6], elementIndices[2], _coord, _ccw );
           normals[elementIndices[1]] += norm;
           normals[elementIndices[5]] += norm;
           normals[elementIndices[6]] += norm;
           normals[elementIndices[2]] += norm;
 
-          norm= hexaNormal ( elementIndices[2], elementIndices[6], elementIndices[7], elementIndices[3], coord, ccw );
+          norm= hexaNormal ( elementIndices[2], elementIndices[6], elementIndices[7], elementIndices[3], _coord, _ccw );
           normals[elementIndices[2]] += norm;
           normals[elementIndices[6]] += norm;
           normals[elementIndices[7]] += norm;
           normals[elementIndices[3]] += norm;
 
-          norm= hexaNormal ( elementIndices[3], elementIndices[7], elementIndices[4], elementIndices[0], coord, ccw );
+          norm= hexaNormal ( elementIndices[3], elementIndices[7], elementIndices[4], elementIndices[0], _coord, _ccw );
           normals[elementIndices[ 3 ]] += norm;
           normals[elementIndices[ 7 ]] += norm;
           normals[elementIndices[ 4 ]] += norm;
           normals[elementIndices[ 0 ]] += norm;
 
-          norm= hexaNormal ( elementIndices[5], elementIndices[4], elementIndices[7], elementIndices[6], coord, ccw );
+          norm= hexaNormal ( elementIndices[5], elementIndices[4], elementIndices[7], elementIndices[6], _coord, _ccw );
           normals[elementIndices[ 5 ]] += norm;
           normals[elementIndices[ 4 ]] += norm;
           normals[elementIndices[ 7 ]] += norm;
@@ -396,25 +396,25 @@ X3DNormalNode *IndexedElementSet::AutoNormal::generateNormalsPerVertex(
         ++i ) {
           (*i).normalizeSafe();
       }
-      normal->vector->setValue( normals );
+      _normal->vector->setValue( normals );
     }
-    return normal;
+    return _normal;
 }
 
 X3DNormalNode *IndexedElementSet::AutoNormal::generateNormalsPerFace( 
-  X3DCoordinateNode *coord,
-  const vector< int > &index,
-  bool ccw ) {
-    Normal *normal = new Normal;
-    if( coord ) {
+  X3DCoordinateNode *_coord,
+  const vector< int > &_index,
+  bool _ccw ) {
+    Normal *_normal = new Normal;
+    if( _coord ) {
       vector< Vec3f > normals;
 
-      for( unsigned int i = 0; i < index.size(); ++i ) {
+      for( unsigned int i = 0; i < _index.size(); ++i ) {
 
         std::vector<int> elementIndices;
 
-        for(;  i < index.size() && index[i] != -1; ++i )
-          elementIndices.push_back( index[i] );
+        for(;  i < _index.size() && _index[i] != -1; ++i )
+          elementIndices.push_back( _index[i] );
 
         if( elementIndices.size() == 1 )
         {
@@ -427,21 +427,21 @@ X3DNormalNode *IndexedElementSet::AutoNormal::generateNormalsPerFace(
 
           Vec3f norm;
           // Add a normal for each of the triangle faces in the tetra
-          normals.push_back( triangleNormal ( elementIndices[0], elementIndices[1], elementIndices[2], coord, ccw ) );
-          normals.push_back( triangleNormal ( elementIndices[0], elementIndices[2], elementIndices[3], coord, ccw ) );
-          normals.push_back( triangleNormal ( elementIndices[0], elementIndices[3], elementIndices[1], coord, ccw ) );
-          normals.push_back( triangleNormal ( elementIndices[1], elementIndices[3], elementIndices[2], coord, ccw ) );
+          normals.push_back( triangleNormal ( elementIndices[0], elementIndices[1], elementIndices[2], _coord, _ccw ) );
+          normals.push_back( triangleNormal ( elementIndices[0], elementIndices[2], elementIndices[3], _coord, _ccw ) );
+          normals.push_back( triangleNormal ( elementIndices[0], elementIndices[3], elementIndices[1], _coord, _ccw ) );
+          normals.push_back( triangleNormal ( elementIndices[1], elementIndices[3], elementIndices[2], _coord, _ccw ) );
 
         } else if ( elementIndices.size() == 8 ) {
 
           Vec3f norm;
           // Add a normal for each of the hexa faces in the hexahedra
-          normals.push_back( hexaNormal ( elementIndices[0], elementIndices[1], elementIndices[2], elementIndices[3], coord, ccw ) );
-          normals.push_back( hexaNormal ( elementIndices[0], elementIndices[4], elementIndices[5], elementIndices[1], coord, ccw ) );
-          normals.push_back( hexaNormal ( elementIndices[1], elementIndices[5], elementIndices[6], elementIndices[2], coord, ccw ) );
-          normals.push_back( hexaNormal ( elementIndices[2], elementIndices[6], elementIndices[7], elementIndices[3], coord, ccw ) );
-          normals.push_back( hexaNormal ( elementIndices[3], elementIndices[7], elementIndices[4], elementIndices[0], coord, ccw ) );
-          normals.push_back( hexaNormal ( elementIndices[5], elementIndices[4], elementIndices[7], elementIndices[6], coord, ccw ) );
+          normals.push_back( hexaNormal ( elementIndices[0], elementIndices[1], elementIndices[2], elementIndices[3], _coord, _ccw ) );
+          normals.push_back( hexaNormal ( elementIndices[0], elementIndices[4], elementIndices[5], elementIndices[1], _coord, _ccw ) );
+          normals.push_back( hexaNormal ( elementIndices[1], elementIndices[5], elementIndices[6], elementIndices[2], _coord, _ccw ) );
+          normals.push_back( hexaNormal ( elementIndices[2], elementIndices[6], elementIndices[7], elementIndices[3], _coord, _ccw ) );
+          normals.push_back( hexaNormal ( elementIndices[3], elementIndices[7], elementIndices[4], elementIndices[0], _coord, _ccw ) );
+          normals.push_back( hexaNormal ( elementIndices[5], elementIndices[4], elementIndices[7], elementIndices[6], _coord, _ccw ) );
 
 
         } else {
@@ -452,18 +452,18 @@ X3DNormalNode *IndexedElementSet::AutoNormal::generateNormalsPerFace(
 
       } 
 
-      normal->vector->setValue( normals );
+      _normal->vector->setValue( normals );
     }
-    return normal;
+    return _normal;
 }
 
-Vec3f IndexedElementSet::AutoNormal::triangleNormal ( int a, int b, int c, X3DCoordinateNode* coord, bool ccw ) {
+Vec3f IndexedElementSet::AutoNormal::triangleNormal ( int a, int b, int c, X3DCoordinateNode* _coord, bool _ccw ) {
   Vec3f norm, A, B, C, AB, BC;
 
   // calculate a normal for the triangle
-  A = coord->getCoord( a );
-  B = coord->getCoord( b );
-  C = coord->getCoord( c );
+  A = _coord->getCoord( a );
+  B = _coord->getCoord( b );
+  C = _coord->getCoord( c );
 
   AB = B - A;
   BC = C - B;
@@ -476,19 +476,19 @@ Vec3f IndexedElementSet::AutoNormal::triangleNormal ( int a, int b, int c, X3DCo
     norm = Vec3f( 1, 0, 0 );
   }
 
-  if( !ccw ) 
+  if( !_ccw ) 
     norm = -norm;
 
   return norm;
 }
 // d does not have to be given assuming the hexa is verified.
-Vec3f IndexedElementSet::AutoNormal::hexaNormal ( int a, int b, int c, int d, X3DCoordinateNode* coord, bool ccw ) {
+Vec3f IndexedElementSet::AutoNormal::hexaNormal ( int a, int b, int c, int d, X3DCoordinateNode* _coord, bool _ccw ) {
   Vec3f norm, A, B, C, AB, BC;
 
   // calculate a normal for the hexa
-  A = coord->getCoord( a );
-  B = coord->getCoord( b );
-  C = coord->getCoord( c );
+  A = _coord->getCoord( a );
+  B = _coord->getCoord( b );
+  C = _coord->getCoord( c );
 
   AB = B - A;
   BC = C - B;
@@ -501,7 +501,7 @@ Vec3f IndexedElementSet::AutoNormal::hexaNormal ( int a, int b, int c, int d, X3
     norm = Vec3f( 1, 0, 0 );
   }
 
-  if( !ccw ) 
+  if( !_ccw ) 
     norm = -norm;
 
   return norm;
