@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -40,8 +40,8 @@ H3DNodeDatabase IndexedTetraSet::database( "IndexedTetraSet",
                                             &X3DComposedGeometryNode::database);
 
 namespace IndexedTetraSetInternals {
-  FIELDDB_ELEMENT( IndexedTetraSet, index, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( IndexedTetraSet, renderMode, INPUT_OUTPUT );
+  FIELDDB_ELEMENT( IndexedTetraSet, index, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( IndexedTetraSet, renderMode, INPUT_OUTPUT )
 }
 
 IndexedTetraSet::IndexedTetraSet (
@@ -79,7 +79,8 @@ IndexedTetraSet::IndexedTetraSet (
     vboFieldsUpToDate( new Field ),
     vbo_id( NULL ),
     autoTangent( new AutoTangent ),
-    render_tangents( false )
+    render_tangents( false ),
+    tex_coord_node( NULL )
 {
   // init fields
   type_name = "IndexedTetraSet";
@@ -283,7 +284,6 @@ void IndexedTetraSet::render ()
           glBindBufferARB( GL_ELEMENT_ARRAY_BUFFER_ARB, *vbo_id );
          
           if( render_mode == "LINES_ADJACENCY" ) {
-            const vector<int> &indices = index->getValue();
             glBufferDataARB( GL_ELEMENT_ARRAY_BUFFER_ARB,
                              indices.size() * sizeof(GLuint),
                              &(*(indices.begin()) ), GL_STATIC_DRAW_ARB );
@@ -365,7 +365,6 @@ void IndexedTetraSet::render ()
         }
 
         if( render_mode == "LINES_ADJACENCY" ) {
-          const vector<int> &indices = index->getValue();
           glDrawRangeElements( GL_LINES_ADJACENCY_EXT,
                                0,
                                coordinate_node->nrAvailableCoords() - 1,
@@ -411,7 +410,6 @@ void IndexedTetraSet::render ()
 
       
       if( render_mode == "LINES_ADJACENCY" ) {
-        const vector<int> &indices = index->getValue();
         glBegin( GL_LINES_ADJACENCY_EXT );
         for ( unsigned int i= 0; i < nr_tetra*4; ++i ) {
 
@@ -490,78 +488,78 @@ void IndexedTetraSet::renderTriangle ( unsigned int a, unsigned int b, unsigned
   renderVertex ( c );
 }
 
-void IndexedTetraSet::renderVertex ( unsigned int index )
+void IndexedTetraSet::renderVertex ( unsigned int _index )
 {
   if( render_tangents ) {
     for( unsigned int attrib_index = 0;
       attrib_index < autoTangent->size(); ++attrib_index ) {
         X3DVertexAttributeNode *attr = 
           autoTangent->getValueByIndex( attrib_index );
-        if( attr ) attr->render( index );
+        if( attr ) attr->render( _index );
     }
   }
-  if( color_node ) color_node->render( index );
+  if( color_node ) color_node->render( _index );
   if( tex_coords_per_vertex )
-    renderTexCoord( index, tex_coord_node );
-  if( fog_coord_node) fog_coord_node->render(index);
+    renderTexCoord( _index, tex_coord_node );
+  if( fog_coord_node) fog_coord_node->render(_index);
   for( unsigned int attrib_index = 0;
     attrib_index < attrib->size(); ++attrib_index ) {
       X3DVertexAttributeNode *attr = 
         attrib->getValueByIndex( attrib_index );
-      if( attr ) attr->render( index );
+      if( attr ) attr->render( _index );
   }
-  coordinate_node->render( index );
+  coordinate_node->render( _index );
 }
 
 void IndexedTetraSet::AutoNormal::update() {
   bool normals_per_vertex = 
     static_cast< SFBool * >( routes_in[0] )->getValue();
-  X3DCoordinateNode *coord = 
+  X3DCoordinateNode *_coord = 
     static_cast< X3DCoordinateNode * >( static_cast< SFCoordinateNode * >
     ( routes_in[1] )->getValue() );
-  const vector<int> &index = 
+  const vector<int> &_index = 
     static_cast< MFInt32 * >( routes_in[2] )->getValue();
-  bool ccw = static_cast< SFBool * >( routes_in[3] )->getValue();
+  bool _ccw = static_cast< SFBool * >( routes_in[3] )->getValue();
 
   if( normals_per_vertex ) 
-    value = generateNormalsPerVertex( coord, index, ccw );
+    value = generateNormalsPerVertex( _coord, _index, _ccw );
   else
-    value = generateNormalsPerFace( coord, index, ccw );
+    value = generateNormalsPerFace( _coord, _index, _ccw );
 }
 
 X3DNormalNode *IndexedTetraSet::AutoNormal::generateNormalsPerVertex( 
-  X3DCoordinateNode *coord,
-  const vector< int > &index,
-  bool ccw ) {
-    Normal *normal = new Normal;
-    if( coord ) {
-      vector< Vec3f > normals( coord->nrAvailableCoords(), 
+  X3DCoordinateNode *_coord,
+  const vector< int > &_index,
+  bool _ccw ) {
+    Normal *_normal = new Normal;
+    if( _coord ) {
+      vector< Vec3f > normals( _coord->nrAvailableCoords(), 
         Vec3f( 0, 0, 0 ) );
-      for( unsigned int i = 0; i < index.size(); i+=4 ) {
+      for( unsigned int i = 0; i < _index.size(); i+=4 ) {
         
         // Check for valid tetra (i.e. 4 indices)
-        if ( i+3 < index.size() ) {
+        if ( i+3 < _index.size() ) {
           Vec3f norm;
 
-          norm= triangleNormal ( index[i], index[i+1], index[i+2], coord, ccw );
-          normals[index[ i ]] += norm;
-          normals[index[ i+1 ]] += norm;
-          normals[index[ i+2 ]] += norm;
+          norm= triangleNormal ( _index[i], _index[i+1], _index[i+2], _coord, _ccw );
+          normals[_index[ i ]] += norm;
+          normals[_index[ i+1 ]] += norm;
+          normals[_index[ i+2 ]] += norm;
 
-          norm= triangleNormal ( index[i], index[i+2], index[i+3], coord, ccw );
-          normals[index[ i ]] += norm;
-          normals[index[ i+2 ]] += norm;
-          normals[index[ i+3 ]] += norm;
+          norm= triangleNormal ( _index[i], _index[i+2], _index[i+3], _coord, _ccw );
+          normals[_index[ i ]] += norm;
+          normals[_index[ i+2 ]] += norm;
+          normals[_index[ i+3 ]] += norm;
 
-          norm= triangleNormal ( index[i], index[i+3], index[i+1], coord, ccw );
-          normals[index[ i ]] += norm;
-          normals[index[ i+3 ]] += norm;
-          normals[index[ i+1 ]] += norm;
+          norm= triangleNormal ( _index[i], _index[i+3], _index[i+1], _coord, _ccw );
+          normals[_index[ i ]] += norm;
+          normals[_index[ i+3 ]] += norm;
+          normals[_index[ i+1 ]] += norm;
 
-          norm= triangleNormal ( index[i+1], index[i+3], index[i+2], coord, ccw );
-          normals[index[ i+1 ]] += norm;
-          normals[index[ i+3 ]] += norm;
-          normals[index[ i+2 ]] += norm;
+          norm= triangleNormal ( _index[i+1], _index[i+3], _index[i+2], _coord, _ccw );
+          normals[_index[ i+1 ]] += norm;
+          normals[_index[ i+3 ]] += norm;
+          normals[_index[ i+2 ]] += norm;
         }
       }
 
@@ -570,21 +568,21 @@ X3DNormalNode *IndexedTetraSet::AutoNormal::generateNormalsPerVertex(
         ++i ) {
           (*i).normalizeSafe();
       }
-      normal->vector->setValue( normals );
+      _normal->vector->setValue( normals );
     }
-    return normal;
+    return _normal;
 }
 
 X3DNormalNode *IndexedTetraSet::AutoNormal::generateNormalsPerFace( 
-  X3DCoordinateNode *coord,
-  const vector< int > &index,
-  bool ccw ) {
-    Normal *normal = new Normal;
-    if( coord ) {
+  X3DCoordinateNode *_coord,
+  const vector< int > &_index,
+  bool _ccw ) {
+    Normal *_normal = new Normal;
+    if( _coord ) {
       vector< Vec3f > normals;
-      for( size_t i = 0; i < index.size(); i+=4 ) {
+      for( size_t i = 0; i < _index.size(); i+=4 ) {
         // make sure we have a valid face. If not use dummy normals. 
-        if( i+3 >= index.size() ) {
+        if( i+3 >= _index.size() ) {
           Vec3f norm =  Vec3f( 1, 0, 0 );
           normals.push_back( norm );
           normals.push_back( norm );
@@ -592,24 +590,24 @@ X3DNormalNode *IndexedTetraSet::AutoNormal::generateNormalsPerFace(
           normals.push_back( norm );
         } else {
           // Add a normal for each of the triangle faces in the tetra
-          normals.push_back( triangleNormal ( index[i], index[i+1], index[i+2], coord, ccw ) );
-          normals.push_back( triangleNormal ( index[i], index[i+2], index[i+3], coord, ccw ) );
-          normals.push_back( triangleNormal ( index[i], index[i+3], index[i+1], coord, ccw ) );
-          normals.push_back( triangleNormal ( index[i+1], index[i+3], index[i+2], coord, ccw ) );
+          normals.push_back( triangleNormal ( _index[i], _index[i+1], _index[i+2], _coord, _ccw ) );
+          normals.push_back( triangleNormal ( _index[i], _index[i+2], _index[i+3], _coord, _ccw ) );
+          normals.push_back( triangleNormal ( _index[i], _index[i+3], _index[i+1], _coord, _ccw ) );
+          normals.push_back( triangleNormal ( _index[i+1], _index[i+3], _index[i+2], _coord, _ccw ) );
         }
       }
-      normal->vector->setValue( normals );
+      _normal->vector->setValue( normals );
     }
-    return normal;
+    return _normal;
 }
 
-Vec3f IndexedTetraSet::AutoNormal::triangleNormal ( int a, int b, int c, X3DCoordinateNode* coord, bool ccw ) {
+Vec3f IndexedTetraSet::AutoNormal::triangleNormal ( int a, int b, int c, X3DCoordinateNode* _coord, bool _ccw ) {
   Vec3f norm, A, B, C, AB, BC;
 
   // calculate a normal for the triangle
-  A = coord->getCoord( a );
-  B = coord->getCoord( b );
-  C = coord->getCoord( c );
+  A = _coord->getCoord( a );
+  B = _coord->getCoord( b );
+  C = _coord->getCoord( c );
 
   AB = B - A;
   BC = C - B;
@@ -622,7 +620,7 @@ Vec3f IndexedTetraSet::AutoNormal::triangleNormal ( int a, int b, int c, X3DCoor
     norm = Vec3f( 1, 0, 0 );
   }
 
-  if( !ccw ) 
+  if( !_ccw ) 
     norm = -norm;
 
   return norm;
@@ -658,10 +656,10 @@ void IndexedTetraSet::getTriangleIndices(
 void IndexedTetraSet::AutoTangent::update() {
   bool normals_per_vertex = 
     static_cast< SFBool * >( routes_in[0] )->getValue();
-  X3DCoordinateNode *coord = 
+  X3DCoordinateNode *_coord = 
     static_cast< X3DCoordinateNode * >( static_cast< SFCoordinateNode * >
     ( routes_in[1] )->getValue() );
-  const vector<int> &index = 
+  const vector<int> &_index = 
     static_cast< MFInt32 * >( routes_in[2] )->getValue();
   X3DTextureCoordinateNode *tex_coord = 
     static_cast< X3DTextureCoordinateNode * >( static_cast< SFTextureCoordinateNode * >
@@ -676,15 +674,15 @@ void IndexedTetraSet::AutoTangent::update() {
   FloatVertexAttribute *binormal = static_cast< FloatVertexAttribute * >(value[1]);
   
   if( normals_per_vertex ) 
-    generateTangentsPerVertex( coord, tex_coord, index, tangent, binormal );
+    generateTangentsPerVertex( _coord, tex_coord, _index, tangent, binormal );
   else
-    generateTangentsPerFace( coord, tex_coord, index, tangent, binormal );
+    generateTangentsPerFace( _coord, tex_coord, _index, tangent, binormal );
 }
 
 void IndexedTetraSet::AutoTangent::generateTangentsPerVertex( 
-                                                           X3DCoordinateNode *coord,
+                                                           X3DCoordinateNode *_coord,
                                                            X3DTextureCoordinateNode *tex_coord,
-                                                           const vector< int > &index,
+                                                           const vector< int > &_index,
                                                            FloatVertexAttribute *tangent_node,
                                                            FloatVertexAttribute *binormal_node) {
   tangent_node->name->setValue( "tangent" );
@@ -694,30 +692,30 @@ void IndexedTetraSet::AutoTangent::generateTangentsPerVertex(
   binormal_node->numComponents->setValue( 3 );
   binormal_node->value->clear();
 
-  if( coord ) {
+  if( _coord ) {
     // the tangent and binormal for a vertex is the average of the tangent
     // of all triangles sharing that vertex.
-    vector< H3DFloat > tangents( coord->nrAvailableCoords() * 3, 0 );
-    vector< H3DFloat > binormals( coord->nrAvailableCoords() * 3, 0 );
-    for( unsigned int j = 0; j < index.size(); j+=4 ) {
+    vector< H3DFloat > tangents( _coord->nrAvailableCoords() * 3, 0 );
+    vector< H3DFloat > binormals( _coord->nrAvailableCoords() * 3, 0 );
+    for( unsigned int j = 0; j < _index.size(); j+=4 ) {
       // make sure we have a valid face. If not use a dummy tangent.
-      if( j+3 >= index.size() ) {
-        tangents[ index[j]*3 ] += 0;
-        tangents[ index[j]*3+1 ] += 1;
-        tangents[ index[j]*3+2 ] += 0;
+      if( j+3 >= _index.size() ) {
+        tangents[ _index[j]*3 ] += 0;
+        tangents[ _index[j]*3+1 ] += 1;
+        tangents[ _index[j]*3+2 ] += 0;
 
-        binormals[ index[j]*3   ] += 0;
-        binormals[ index[j]*3+1 ] += 0;
-        binormals[ index[j]*3+2 ] += 1;
+        binormals[ _index[j]*3   ] += 0;
+        binormals[ _index[j]*3+1 ] += 0;
+        binormals[ _index[j]*3+2 ] += 1;
       } else {
-        calculateAndAddTangentPerVertex( coord, tex_coord, tangents, binormals,
-                                         index[j], index[j+1], index[j+2] );
-        calculateAndAddTangentPerVertex( coord, tex_coord, tangents, binormals,
-                                         index[j], index[j+2], index[j+3] );
-        calculateAndAddTangentPerVertex( coord, tex_coord, tangents, binormals,
-                                         index[j], index[j+3], index[j+1] );
-        calculateAndAddTangentPerVertex( coord, tex_coord, tangents, binormals,
-                                         index[j+1], index[j+3], index[j+2] );
+        calculateAndAddTangentPerVertex( _coord, tex_coord, tangents, binormals,
+                                         _index[j], _index[j+1], _index[j+2] );
+        calculateAndAddTangentPerVertex( _coord, tex_coord, tangents, binormals,
+                                         _index[j], _index[j+2], _index[j+3] );
+        calculateAndAddTangentPerVertex( _coord, tex_coord, tangents, binormals,
+                                         _index[j], _index[j+3], _index[j+1] );
+        calculateAndAddTangentPerVertex( _coord, tex_coord, tangents, binormals,
+                                         _index[j+1], _index[j+3], _index[j+2] );
       }
     }
     
@@ -740,9 +738,9 @@ void IndexedTetraSet::AutoTangent::generateTangentsPerVertex(
 }
 
 void IndexedTetraSet::AutoTangent::generateTangentsPerFace( 
-                                                              X3DCoordinateNode *coord,
+                                                              X3DCoordinateNode *_coord,
                                                               X3DTextureCoordinateNode *tex_coord,
-                                                              const vector< int > &index,
+                                                              const vector< int > &_index,
                                                               FloatVertexAttribute *tangent_node,
                                                               FloatVertexAttribute *binormal_node ) {
 
@@ -753,16 +751,16 @@ void IndexedTetraSet::AutoTangent::generateTangentsPerFace(
   binormal_node->numComponents->setValue( 3 );
   binormal_node->value->clear();
 
-  if( coord ) {
+  if( _coord ) {
     vector< float > tangents;
     vector< float > binormals;
 
-    tangents.reserve( 4 * index.size() / 4 );
-    binormals.reserve( 4 * index.size() / 4 );
+    tangents.reserve( 4 * _index.size() / 4 );
+    binormals.reserve( 4 * _index.size() / 4 );
 
-    for( size_t j = 0; j < index.size(); j+=4 ) {
+    for( size_t j = 0; j < _index.size(); j+=4 ) {
       // make sure we have a valid face. If not use a dummy normal.
-      if( j+3 >= index.size() ) {
+      if( j+3 >= _index.size() ) {
         tangents.push_back( 0 );
         tangents.push_back( 1 );
         tangents.push_back( 0 );
@@ -772,14 +770,14 @@ void IndexedTetraSet::AutoTangent::generateTangentsPerFace(
         binormals.push_back( 1 );
       } else {  
         // calculate a tangent
-        calculateAndAddTangentPerFace( coord, tex_coord, tangents, binormals,
-                                       index[j], index[j+1], index[j+2] );
-        calculateAndAddTangentPerFace( coord, tex_coord, tangents, binormals,
-                                       index[j], index[j+2], index[j+3] );
-        calculateAndAddTangentPerFace( coord, tex_coord, tangents, binormals,
-                                       index[j], index[j+3], index[j+1] );
-        calculateAndAddTangentPerFace( coord, tex_coord, tangents, binormals,
-                                       index[j+1], index[j+3], index[j+2] );
+        calculateAndAddTangentPerFace( _coord, tex_coord, tangents, binormals,
+                                       _index[j], _index[j+1], _index[j+2] );
+        calculateAndAddTangentPerFace( _coord, tex_coord, tangents, binormals,
+                                       _index[j], _index[j+2], _index[j+3] );
+        calculateAndAddTangentPerFace( _coord, tex_coord, tangents, binormals,
+                                       _index[j], _index[j+3], _index[j+1] );
+        calculateAndAddTangentPerFace( _coord, tex_coord, tangents, binormals,
+                                       _index[j+1], _index[j+3], _index[j+2] );
       }
     }
 
@@ -790,12 +788,12 @@ void IndexedTetraSet::AutoTangent::generateTangentsPerFace(
 
 
 
-Vec3f IndexedTetraSet::AutoTangent::getTexCoord( X3DCoordinateNode *coord,
+Vec3f IndexedTetraSet::AutoTangent::getTexCoord( X3DCoordinateNode *_coord,
                                                     X3DTextureCoordinateNode *tex_coord,
-                                                    int index ) {
+                                                    int _index ) {
   if( tex_coord ) {
     if( tex_coord->supportsGetTexCoord( 0 ) ) {
-      Vec4f tc = tex_coord->getTexCoord( index, 0 );
+      Vec4f tc = tex_coord->getTexCoord( _index, 0 );
       return Vec3f( tc.x, tc.y, tc.z ) / tc.w;
     } else {
       Console(4) << "Warning: X3DTextureCoordinateNode does not support getTexCoord() function. Tangents and binormals cannot be calculated for IndexedTetraSet." << endl;
@@ -803,7 +801,7 @@ Vec3f IndexedTetraSet::AutoTangent::getTexCoord( X3DCoordinateNode *coord,
   } else {
     IndexedTetraSet *its = static_cast< IndexedTetraSet * >( getOwner() );
     Matrix4f to_str = its->getDefaultTexGenMatrix();
-    return to_str * coord->getCoord( index );
+    return to_str * _coord->getCoord( _index );
   }
 
   return Vec3f(0,0,0);
@@ -854,20 +852,20 @@ void IndexedTetraSet::AutoTangent::calculateTangent( const Vec3f &a, const Vec3f
 
 
 void IndexedTetraSet::AutoTangent::calculateAndAddTangentPerVertex(
-  X3DCoordinateNode *coord,
+  X3DCoordinateNode *_coord,
   X3DTextureCoordinateNode *tex_coord,
   vector< H3DFloat > &tangents,
   vector< H3DFloat > &binormals,
   int i, int j, int k ) {
   Vec3f tangent, binormal;
   // calculate a tangent
-  Vec3f a = coord->getCoord( i );
-  Vec3f b = coord->getCoord( j );
-  Vec3f c = coord->getCoord( k );
+  Vec3f a = _coord->getCoord( i );
+  Vec3f b = _coord->getCoord( j );
+  Vec3f c = _coord->getCoord( k );
 
-  Vec3f ta = getTexCoord( coord, tex_coord, i );
-  Vec3f tb = getTexCoord( coord, tex_coord, j );
-  Vec3f tc = getTexCoord( coord, tex_coord, k );
+  Vec3f ta = getTexCoord( _coord, tex_coord, i );
+  Vec3f tb = getTexCoord( _coord, tex_coord, j );
+  Vec3f tc = getTexCoord( _coord, tex_coord, k );
 
   calculateTangent( a, b, c,
                     ta, tb, tc,
@@ -895,20 +893,20 @@ void IndexedTetraSet::AutoTangent::calculateAndAddTangentPerVertex(
 }
 
 void IndexedTetraSet::AutoTangent::calculateAndAddTangentPerFace(
-  X3DCoordinateNode *coord,
+  X3DCoordinateNode *_coord,
   X3DTextureCoordinateNode *tex_coord,
   vector< H3DFloat > &tangents,
   vector< H3DFloat > &binormals,
   int i, int j, int k ) {
   Vec3f tangent, binormal;
   // calculate a tangent
-  Vec3f a = coord->getCoord( i );
-  Vec3f b = coord->getCoord( j );
-  Vec3f c = coord->getCoord( k );
+  Vec3f a = _coord->getCoord( i );
+  Vec3f b = _coord->getCoord( j );
+  Vec3f c = _coord->getCoord( k );
 
-  Vec3f ta = getTexCoord( coord, tex_coord, i );
-  Vec3f tb = getTexCoord( coord, tex_coord, j );
-  Vec3f tc = getTexCoord( coord, tex_coord, k );
+  Vec3f ta = getTexCoord( _coord, tex_coord, i );
+  Vec3f tb = getTexCoord( _coord, tex_coord, j );
+  Vec3f tc = getTexCoord( _coord, tex_coord, k );
 
   calculateTangent( a, b, c,
                     ta, tb, tc,
