@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -114,7 +114,7 @@ PeriodicThread::CallbackCode ODECallbacks::doSimulationSteps(void *data) {
       transforms[index+15] = 1;
 
       if( index == 0 ) {
-        dReal* t = dGeomTriMeshGetLastTransform( i->second );
+        const dReal* real_t = dGeomTriMeshGetLastTransform( i->second );
       }
     }
 
@@ -152,6 +152,9 @@ PeriodicThread::CallbackCode ODECallbacks::doSimulationSteps(void *data) {
   return PeriodicThread::CALLBACK_CONTINUE;
 }
 
+PeriodicThread::CallbackCode ODECallbacks::synchroniseWithSceneGraph(void *data) {
+  return PeriodicThread::CALLBACK_DONE;
+}
 
 PeriodicThread::CallbackCode ODECallbacks::setWorldParameters(void *data) {
   PhysicsEngineParameters::WorldParameters *params = static_cast< PhysicsEngineParameters::WorldParameters *>( data );
@@ -249,9 +252,6 @@ void ODECallbacks::nearCallback(void *data, dGeomID o1, dGeomID o2){
         for( vector< string >::iterator i = p.applied_parameters.begin();
             i != p.applied_parameters.end(); ++i ) {
           if( (*i) == "BOUNCE" ) mode |= dContactBounce;
-          // TODO: we do not have fdir in CollisionCollection. Why is this?
-          // This mode is useless then
-          //else if( (*i) == "USER_FRICTION" ) mode |= dContactFDir1;
           else if( (*i) == "FRICTION_COEFFICIENT-2" ) mode |= dContactMu2;
           else if( (*i) == "ERROR_CORRECTION" ) mode |= dContactSoftERP;
           else if( (*i) == "CONSTANT_FORCE" ) mode |= dContactSoftCFM;
@@ -308,9 +308,6 @@ void ODECallbacks::nearCallback(void *data, dGeomID o1, dGeomID o2){
     for( vector< string >::iterator i = p.applied_parameters.begin();
          i != p.applied_parameters.end(); ++i ) {
       if( (*i) == "BOUNCE" ) mode |= dContactBounce;
-      // TODO: we do not have fdir in CollisionCollection. Why is this?
-      // This mode is useless then
-      //else if( (*i) == "USER_FRICTION" ) mode |= dContactFDir1;
       else if( (*i) == "FRICTION_COEFFICIENT-2" ) mode |= dContactMu2;
       else if( (*i) == "ERROR_CORRECTION" ) mode |= dContactSoftERP;
       else if( (*i) == "CONSTANT_FORCE" ) mode |= dContactSoftCFM;
@@ -419,7 +416,6 @@ void ODECallbacks::setRigidBodyParameters( PhysicsEngineParameters::RigidBodyPar
         BoxBound *bb = dynamic_cast< BoxBound * >( g->bound->getValue() );
         Vec3f size( 0.1f, 0.1f, 0.1f );
         if( bb ) size = bb->size->getValue();
-        // TODO: take the position of the bounding box into consideration
         dMassSetBoxTotal( &ode_mass, params->getMass(), size.x, size.y, size.z);
       } else {
         Console(4) << "Invalid node used in mass density model \" " << n->getTypeName() << "\". " 
@@ -496,7 +492,7 @@ void ODECallbacks::setRigidBodyParameters( PhysicsEngineParameters::RigidBodyPar
       if ( fixed ) {
         // Attach the shape to 0 instead of a rigid body. ODE way of saying fixed
         // object.
-        dGeomSetBody( collidable_id, 0 ); 
+        dGeomSetBody( collidable_id, 0 );
     
         // Set geom position and rotation = body position and rotation
         const dReal *pos = dBodyGetPosition( ( dBodyID )params->getBodyId() );
@@ -504,16 +500,19 @@ void ODECallbacks::setRigidBodyParameters( PhysicsEngineParameters::RigidBodyPar
         
         const dReal *R   = dBodyGetRotation( ( dBodyID )params->getBodyId() );
         dMatrix3 rotation;    
-        for ( int i=0; i<12;++i ) {
-          rotation[i] = R[i];
+        for ( int j=0; j<12;++j ) {
+          rotation[j] = R[j];
         }
         dGeomSetRotation( collidable_id, rotation );      
       } else {
         // connect body and shape.
-        dGeomSetBody( collidable_id, ( dBodyID )params->getBodyId() );   
+        dGeomSetBody( collidable_id, ( dBodyID )params->getBodyId() );
       }
     }
   }
+
+  // Once a body is connected / updated the offsets can be applied relative to the body.
+  ODECallbacks::handlePendingOffsets( params );
 }
 
 H3DUtil::PeriodicThread::CallbackCode ODECallbacks::setRigidBodyParameters(void *data) {
@@ -555,32 +554,31 @@ H3DUtil::PeriodicThread::CallbackCode ODECallbacks::getRigidBodyParameters(void
   return PeriodicThread::CALLBACK_DONE;
 }
 
+void updateCollidableParameters( PhysicsEngineParameters::CollidableParameters* params ) {
+
+  dGeomID geom_id = ( dGeomID )params->getCollidableId();
+
+  ODECallbacks::ODESpecificData *ode_data = 
+    static_cast< ODECallbacks::ODESpecificData * >(params->getEngine()->getEngineSpecificData());
+
+  ODECallbacks::ODESpecificData::CollidableOffset offset;
+  offset.rotation = params->getRotation();
+  offset.translation = params->getTranslation();
+
+  ode_data->pending_offsets[ geom_id ].push_back( offset );
+  
+  if( params->getEnabled() )
+    dGeomEnable( geom_id );
+  else
+    dGeomDisable( geom_id );
+}
 
 H3DUtil::PeriodicThread::CallbackCode ODECallbacks::setCollidableParameters(void *data) {
   PhysicsEngineParameters::CollidableParameters *params = 
     static_cast< PhysicsEngineParameters::CollidableParameters *>( data );
-  dGeomID geomID = dGeomTransformGetGeom( ( dGeomID )params->getCollidableId() );
-
-  dMatrix3 R;
-  const Rotation &rotation = params->getRotation();
-  dRFromAxisAndAngle(R, 
-                     rotation.axis.x,  
-                     rotation.axis.y,  
-                     rotation.axis.z, 
-                     rotation.angle);
-
-  const Vec3f &translation = params->getTranslation();
-  dGeomSetPosition( geomID, 
-                   translation.x, 
-                   translation.y, 
-                   translation.z );
-  dGeomSetRotation( geomID, R);
-  
-  if( params->getEnabled() )
-    dGeomEnable( ( dGeomID )params->getCollidableId() );
-  else
-    dGeomDisable( ( dGeomID )params->getCollidableId() );
+  updateCollidableParameters( params );
 
+  delete params;
   return PeriodicThread::CALLBACK_DONE;
 }
 
@@ -594,7 +592,6 @@ H3DUtil::PeriodicThread::CallbackCode ODECallbacks::addCollidable(void *data) {
     addOffset( p );
     //cerr << "ODE: collidableOffet created with collidableID " << params->collidable_id << endl;
   }
-  ODECallbacks::setCollidableParameters( params );
   return PeriodicThread::CALLBACK_DONE;
 }
 
@@ -606,18 +603,68 @@ void ODECallbacks::addOffset( OffsetParameters * params ) {
   } else if ( params->getParentSpaceId() == CollidableParameters::NO_SPACE ) {
     params->setParentSpaceId( (H3DSpaceId) 0 );
   }
-  dGeomID transform = dCreateGeomTransform( (dSpaceID)params->getParentSpaceId() );
-  dGeomTransformSetCleanup( transform, 0 );
-  dGeomTransformSetInfo( transform, 1 );
-  params->setCollidableId( ( H3DCollidableId )transform );
-  dGeomTransformSetGeom( transform, (dGeomID) params->collidable );
-  
-  // Move tri_mesh pointer so that only the currently most parent
-  // collidable-id points at the tri_mesh
-  if( ode_data->tri_meshes.count( (dGeomID) params->collidable ) != 0 ){
-    ode_data->tri_meshes[transform] = ode_data->tri_meshes[(dGeomID) params->collidable];
-    ode_data->tri_meshes.erase((dGeomID) params->collidable);
+
+  params->setCollidableId( params->collidable );
+
+  ODESpecificData::CollidableOffset offset;
+  offset.translation = params->getTranslation();
+  offset.rotation = params->getRotation();
+
+  dGeomID geom_id = (dGeomID) params->collidable;
+
+  ode_data->pending_offsets[ geom_id ].push_back( offset );
+
+  if( params->getEnabled() )
+    dGeomEnable( geom_id );
+  else
+    dGeomDisable( geom_id );
+}
+
+void ODECallbacks::handlePendingOffsets( RigidBodyParameters* params ) {
+
+  ODESpecificData *ode_data = 
+    static_cast< ODESpecificData * >(params->getEngine()->getEngineSpecificData());
+
+  std::vector< H3DCollidableId > geometries = params->getGeometry();
+
+  for( std::vector< H3DCollidableId >::iterator i = geometries.begin(); i != geometries.end(); ++i ) {
+
+    dGeomID geom_id = (dGeomID) *i;
+    dBodyID body_id = dGeomGetBody( geom_id );
+
+    if( body_id != 0 && ode_data->pending_offsets.count( geom_id ) != 0 ) {
+
+      dGeomClearOffset( geom_id );
+
+      const std::vector< ODESpecificData::CollidableOffset >& offsets = ode_data->pending_offsets[ geom_id ];
+
+      Rotation total_rotation = Rotation();
+      Vec3f total_translation = Vec3f();
+
+      for( size_t j = 0; j < offsets.size(); ++j ) {
+        total_translation += offsets[j].translation;
+        total_rotation *= offsets[j].rotation;
+      }
+
+      dMatrix3 R;
+      dRFromAxisAndAngle(R, 
+                          total_rotation.axis.x,  
+                          total_rotation.axis.y,  
+                          total_rotation.axis.z, 
+                          total_rotation.angle );
+
+      dGeomSetOffsetPosition( geom_id, 
+                              total_translation.x, 
+                              total_translation.y, 
+                              total_translation.z );
+      dGeomSetOffsetRotation( geom_id, R);
+
+      ode_data->pending_offsets.erase( geom_id );
+
+    }
+
   }
+
 }
 
 void ODECallbacks::addShape( ShapeParameters * params ) {
@@ -628,35 +675,42 @@ void ODECallbacks::addShape( ShapeParameters * params ) {
   } else if ( params->getParentSpaceId() == CollidableParameters::NO_SPACE ) {
     params->setParentSpaceId( (H3DSpaceId) 0 );
   }
-  dGeomID transform = dCreateGeomTransform( (dSpaceID)params->getParentSpaceId() );
-  dGeomTransformSetCleanup( transform, 1 );
-  dGeomTransformSetInfo( transform, 1 );
 
   dGeomID temp_geom;
 
   if ( Box *b = dynamic_cast< Box * >( params->getShape() )) {
     Vec3f size = b->size->getValue();
-    temp_geom = dCreateBox(0, size.x, size.y, size.z);
+    temp_geom = dCreateBox(ode_data->space_id, size.x, size.y, size.z);
   } else if ( Sphere *sph = dynamic_cast< Sphere * >( params->getShape() )) {
     dReal radius = sph->radius->getValue();
-    temp_geom = dCreateSphere(0, radius);
+    temp_geom = dCreateSphere(ode_data->space_id, radius);
   } else if ( Cylinder *cyl = dynamic_cast< Cylinder * >( params->getShape() )) {  
     dReal radius = cyl->radius->getValue();
     dReal height = cyl->height->getValue();
 
     // the ODE cylinder is specified along z-axis, so we need to rotate
     // it to be along y-axis.
-    temp_geom = dCreateGeomTransform( 0 );
-    dGeomTransformSetCleanup( temp_geom, 1 );
-    dGeomTransformSetInfo( temp_geom, 1 );
-    dGeomID ode_cyl = dCreateCylinder(0, radius, height);
-    dMatrix3 R;
-    dRFromAxisAndAngle(R, 1, 0, 0, Constants::pi/2 );  
-    dGeomSetRotation( ode_cyl, R );
-    dGeomTransformSetGeom( temp_geom, ode_cyl );
+    temp_geom = dCreateCylinder(ode_data->space_id, radius, height);
+
+    ODESpecificData::CollidableOffset offset;
+    offset.rotation = Rotation( Vec3f( 1.0f, 0.0f, 0.0f ), H3DFloat( Constants::pi ) / 2.0f );
+    offset.translation = Vec3f( 0.0f, 0.0f, 0.0f );
+
+    ode_data->pending_offsets[ temp_geom ].push_back( offset );
+
   } else {
     vector< HAPI::Collision::Triangle > triangles;
-    params->getShape()->boundTree->getValue()->getAllTriangles( triangles );
+    if( params->haveShape() ) {    
+      if( params->getShape()->boundTree->isUpToDate() ) {
+        // Only do getValue() if the tree is already uptodate, else causes OpenGL error
+        params->getShape()->boundTree->getValue()->getAllTriangles( triangles );
+      }
+    }
+    // if there are no triangles built then don't create a shape
+    if( triangles.size() > 0 ) {
+      Console( 4 ) << "Error: ODE. error no triangles in the shape bound tree!" << endl;
+    }
+
     dVector3 *ode_tris = new dVector3[ triangles.size()*3];
     dTriIndex *ode_indices = new dTriIndex[ triangles.size() * 3];
     for( unsigned int i = 0; i < triangles.size(); ++i ) {
@@ -678,14 +732,21 @@ void ODECallbacks::addShape( ShapeParameters * params ) {
 
     dTriMeshDataID tri_mesh_id = dGeomTriMeshDataCreate();
     dGeomTriMeshDataBuildSimple( tri_mesh_id,
-                                 (dReal*)ode_tris, triangles.size() * 3,
-                                 ode_indices, triangles.size() * 3 );
-    temp_geom = dCreateTriMesh( 0, //ode_data->space_id,
+                                 (dReal*)ode_tris, static_cast<int>(triangles.size() * 3),
+                                 ode_indices, static_cast<int>(triangles.size() * 3 ));
+    temp_geom = dCreateTriMesh( ode_data->space_id,
                                 tri_mesh_id, NULL, NULL, NULL );
-    ode_data->tri_meshes[transform] = temp_geom;
-  }  
-  params->setCollidableId( ( H3DCollidableId )transform );
-  dGeomTransformSetGeom( transform, temp_geom );
+    ode_data->tri_meshes[temp_geom] = temp_geom;
+  }
+
+  params->setCollidableId( ( H3DCollidableId )temp_geom );
+
+  // Add offset defined by the collidable's initial transformation matrix.
+  ODESpecificData::CollidableOffset offset;
+  offset.rotation = params->getRotation();
+  offset.translation = params->getTranslation();
+  ode_data->pending_offsets[ temp_geom ].push_back( offset );
+
 }
 
 
@@ -708,7 +769,7 @@ H3DUtil::PeriodicThread::CallbackCode ODECallbacks::getCollidableParameters(void
   ODESpecificData *ode_data = 
     static_cast< ODESpecificData * >(params->getEngine()->getEngineSpecificData());
   
-  dGeomID geomID = dGeomTransformGetGeom( ( dGeomID )params->getCollidableId() );
+  dGeomID geomID = ( dGeomID )params->getCollidableId();
   const dReal *pos = dGeomGetPosition( geomID );
   const dReal *R   = dGeomGetRotation( geomID );
   Rotation rot = Rotation(Matrix3f((H3DFloat)R[0], (H3DFloat)R[4], (H3DFloat)R[8], 
@@ -842,6 +903,7 @@ void ODECallbacks::setDoubleAxisHingeJointParameters(void *data) {
     dJointSetHinge2Anchor( ( dJointID )params->getConstraintId(), 
                            ap.x, ap.y, ap.z);
   }
+#ifdef ODE_VERSION_013_OR_EARLIER
   if( params->haveAxis1() ) {
     const Vec3f &axis1 = params->getAxis1();
     dJointSetHinge2Axis1( ( dJointID )params->getConstraintId(), 
@@ -852,6 +914,27 @@ void ODECallbacks::setDoubleAxisHingeJointParameters(void *data) {
     dJointSetHinge2Axis2( ( dJointID )params->getConstraintId(), 
                           axis2.x, axis2.y, axis2.z);
   }
+#else
+  dReal axis1_ode[3] = {0,0,0};
+  if( params->haveAxis1() ) {
+    const Vec3f &axis1 = params->getAxis1();
+    axis1_ode[0] = axis1.x;
+    axis1_ode[1] = axis1.y;
+    axis1_ode[2] = axis1.z;
+  }
+  
+  dReal axis2_ode[3] = {0,0,0};
+  if( params->haveAxis2() ) {
+    const Vec3f &axis2 = params->getAxis2();
+    axis2_ode[0] = axis2.x;
+    axis2_ode[1] = axis2.y;
+    axis2_ode[2] = axis2.z;
+  }
+  
+  if( params->haveAxis1() || params->haveAxis2() ) {
+    dJointSetHinge2Axes( ( dJointID )params->getConstraintId(), params->haveAxis1() ? axis1_ode : NULL, params->haveAxis2() ? axis2_ode : NULL );
+  }
+#endif
   if( params->haveDesiredAngularVelocity1() ) {
     dJointSetHinge2Param( ( dJointID )params->getConstraintId(), dParamVel, params->getDesiredAngularVelocity1() );
   }
@@ -1395,7 +1478,7 @@ H3DUtil::PeriodicThread::CallbackCode ODECallbacks::setConstraintParameters(void
 }
 
 PeriodicThread::CallbackCode ODECallbacks::removeCollidable( void *data ) {
-  CollidableParameters *params = static_cast< CollidableParameters * >( data ); 
+  CollidableParameters *params = static_cast< CollidableParameters * >( data );
   dGeomDestroy( ( dGeomID )params->getCollidableId());
   ODESpecificData *ode_data = 
     static_cast< ODESpecificData * >(params->getEngine()->getEngineSpecificData());
@@ -1451,6 +1534,7 @@ PeriodicThread::CallbackCode ODECallbacks::setSpaceParameters(void *data) {
     static_cast< PhysicsEngineParameters::SpaceParameters *>( data );
   ODESpecificData *ode_data = 
     static_cast< ODESpecificData * >(params->getEngine()->getEngineSpecificData());
+
   if( params->getEnabled() )
     dGeomEnable( ( dGeomID )params->getSpaceId() );
   else
