@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -58,7 +58,7 @@ void PhysXCallbacks::ContactReport::onContactNotify(NxContactPair& pair, NxU32 e
   NxContactStreamIterator i(pair.stream);
   // PhysX reports contacts for deleted actors, we are not interested in those contacts.
   // It does however seems to be some sort of bug in such a way that new actors
-  // will not report contacts after first run of doSimulation. \todo Investigate further.
+  // will not report contacts after first run of doSimulation.
   if( pair.isDeletedActor[0] || pair.isDeletedActor[1] )
     return;
 
@@ -116,13 +116,10 @@ stiffnessUnitType ( PhysicsEngineParameters::MaterialPropertyParameters::UNIT_UN
     generateMeshDesc ( params, vertices_ptr, indices_ptr );
 
   // Cook the mesh
-  // TODO: handle errors
   if ( !cookMesh ( meshDesc ) ) {
     Console(4) << "FAILED to cook mesh" << endl;
   } else {
 
-    // TODO: Destroy the mesh description
-
     // Allocate recieve buffers
     allocateReceiveBuffers(meshDesc.numVertices, meshDesc.numTetrahedra);
 
@@ -191,8 +188,6 @@ void PhysXSoftBody::setParameters ( H3DSoftBodyNodeParameters& params ) {
         params.getH3DPhysicsMaterialParameters();
     if( matPar->haveMass() &&
         matPar->haveMassParameters() ) {
-      // TODO: Does the cloth have a mass?
-      //mass= bodyParameters.getMass();
     }
     if( matPar->haveStiffness() &&
         matPar->haveStiffnessParameters() ) {
@@ -364,7 +359,6 @@ void PhysXSoftBody::applyStiffnessParameters ( StiffnessParameters& stiffnessPar
 
   switch ( stiffnessUnitType ) {
   case MaterialPropertyParameters::UNIT_UNIFORM:
-    // TODO: Bullet equivelant conversion factor
     softBody->setStretchingStiffness ( stiffnessParams.getValuePerUnit(0)*100 );
     break;
   default:
@@ -395,11 +389,9 @@ stiffnessUnitType ( PhysicsEngineParameters::MaterialPropertyParameters::UNIT_UN
                                                indices_ptr );
 
   // Cook the mesh
-  // TODO: handle errors
   cout << "Cooking the mesh descrition" << endl;
   if ( !cookMesh ( meshDesc ) ) {
     cerr << "FAILED to cook mesh" << endl;
-    // TODO: Destroy the mesh description
   } else {
 
     // Allocate recieve buffers
@@ -472,8 +464,6 @@ void PhysXCloth::setParameters ( H3DSoftBodyNodeParameters& params ) {
         params.getH3DPhysicsMaterialParameters();
     if( matPar->haveMass() &&
         matPar->haveMassParameters() ) {
-      // TODO: Does the cloth have a mass?
-      //mass= bodyParameters.getMass();
     }
     if( matPar->haveStiffness() &&
         matPar->haveStiffnessParameters() ) {
@@ -666,7 +656,6 @@ void PhysXCloth::applyStiffnessParameters ( StiffnessParameters& stiffnessParams
 
   switch ( stiffnessUnitType ) {
   case MaterialPropertyParameters::UNIT_UNIFORM:
-    // TODO: Bullet equivelant conversion factor
     cloth->setStretchingStiffness ( stiffnessParams.getValuePerUnit(0)*100 );
     break;
   default:
@@ -711,7 +700,6 @@ PeriodicThread::CallbackCode PhysXCallbacks::initEngine( void *data ) {
 }
 
 
-// deInitEngine
 PeriodicThread::CallbackCode PhysXCallbacks::deInitEngine( void *data ) {
   PhysicsEngineThread *pt = static_cast< PhysicsEngineThread *>( data );
   PhysXSpecificData *physx_data = static_cast< PhysXSpecificData * >( pt->getEngineSpecificData() );
@@ -729,8 +717,6 @@ PeriodicThread::CallbackCode PhysXCallbacks::deInitEngine( void *data ) {
   return PeriodicThread::CALLBACK_DONE;
 }
 
-// TODO verify this function
-// doSimulationSteps()
 PeriodicThread::CallbackCode PhysXCallbacks::doSimulationSteps(void *data) {
   PhysicsEngineThread * physics_thread = 
     static_cast< PhysicsEngineThread * >( data );
@@ -779,6 +765,9 @@ PeriodicThread::CallbackCode PhysXCallbacks::doSimulationSteps(void *data) {
   return PeriodicThread::CALLBACK_CONTINUE;
 }
 
+PeriodicThread::CallbackCode PhysXCallbacks::synchroniseWithSceneGraph(void *data) {
+  return PeriodicThread::CALLBACK_DONE;
+}
 
 // setWorldParams()
 PeriodicThread::CallbackCode PhysXCallbacks::setWorldParameters( void *data ) {
@@ -820,8 +809,7 @@ PeriodicThread::CallbackCode PhysXCallbacks::setWorldParameters( void *data ) {
       }
     }
   }
-  // TODO: take care of all field values below
-  // constantForceMix
+
   if( params->haveConstantForceMix() ) {
   }
   // contactSurfaceThickness
@@ -904,7 +892,6 @@ PeriodicThread::CallbackCode PhysXCallbacks::removeRigidBody( void *data ) {
            i != bodies.end(); ++i ) {
       if ( *i == params->getBodyId() ) {
         PhysXCallbacks::releaseActor( physx_data, (NxActor*)params->getBodyId() );
-        // TODO: note how releaseActor will affect joints in PhysX. refer to releaseActor() doc
       }
     }
   }
@@ -925,7 +912,6 @@ void PhysXCallbacks::setRigidBodyParameters(
       actor->setSleepLinearVelocity( -1 );
       actor->setSleepAngularVelocity( -1 );
     }
-    // TODO: disableTime
   }
 
   // NOTE: autoDamp, angular/linear damping are handled in saveUpdateRate of PET
@@ -954,8 +940,6 @@ void PhysXCallbacks::setRigidBodyParameters(
         NxVec3 inertia;
         NxComputeSphereInertiaTensor( inertia, params->getMass(), sph->radius->getValue(), false );
         actor->setMassSpaceInertiaTensor( inertia );
-      } else if ( Cone *con = dynamic_cast< Cone* >( n ) ) {
-        // TODO No direct Cone inertia tensor calculation in PhysX
       }
     } else {
       // massDensityModel is NULL, use user-given values instead
@@ -963,7 +947,6 @@ void PhysXCallbacks::setRigidBodyParameters(
       actor->setCMassOffsetLocalPosition( NxVec3( params->getCenterOfMass().x,
                                                   params->getCenterOfMass().y,
                                                   params->getCenterOfMass().z ) );
-      // TODO: set inertia in body frame
     }
   }
   // start position
@@ -980,8 +963,7 @@ void PhysXCallbacks::setRigidBodyParameters(
     quat.fromAngleAxisFast( start_ort.angle, r);
     actor->setGlobalOrientation( NxMat33( quat ) );
   }
-  // TODO: this was set to CollisionShape translation before. << ??
- 
+
   if ( params->havePosition() || params->haveOrientation() ) {
     actor->raiseBodyFlag ( NX_BF_KINEMATIC );
 
@@ -1031,8 +1013,6 @@ void PhysXCallbacks::setRigidBodyParameters(
           NxSphereShapeDesc sph_desc;
           sph->saveToDesc( sph_desc );
           new_actor_desc.shapes.push_back( &sph_desc );
-        } else {
-          // TODO: other shapes
         }
         // release old shape
         actor->releaseShape( *(*si)->shape );
@@ -1077,7 +1057,6 @@ void PhysXCallbacks::setRigidBodyParameters(
           sph->saveToDesc( sph_desc );
           new_shape = actor->createShape( sph_desc );
         } else {
-          // TODO: case when it is neither box nor sphere
           continue;
         }
         assert( new_shape != NULL );
@@ -1174,10 +1153,13 @@ H3DUtil::PeriodicThread::CallbackCode PhysXCallbacks::addCollidable( void *data
       Vec3f size = box->size->getValue();
       desc.dimensions = NxVec3( (NxReal)0.5*size.x, (NxReal)0.5*size.y, (NxReal)0.5*size.z );
     } else {
-      // TODO add more supported shapes.
       Console(3) << "Warning: Only Boxes and Sphere collidable shapes are supported by PhysX."
-                 << " Current shape is of type " << params->getShape()->getTypeName()
-                 << " undesired behaviour will occur." << endl;
+                 << " Current shape is of type ";
+      if( params->haveShape() )
+        Console(3) << params->getShape()->getTypeName();
+      else
+        Console(3) << " NULL ";
+      Console(3) << " undesired behaviour will occur." << endl;
       if( H3DBoundedObject * bound = dynamic_cast< H3DBoundedObject * >( params->getShape() ) ) {
         if( BoxBound * box_bound = dynamic_cast< BoxBound * > ( bound->bound->getValue() ) ) {
           Vec3f size = box_bound->size->getValue();
@@ -1480,10 +1462,8 @@ void PhysXCallbacks::setSingleAxisHingeJointParameters( void *data ) {
     joint->setLimits( limit );
   }
   if( params->haveStopBounce() ) {
-    // TODO
   }
   if( params->haveStopErrorCorrection() ) {
-    // TODO
   }
 }
 
@@ -1558,10 +1538,8 @@ void PhysXCallbacks::setSliderJointParameters( void *data ) {
     joint->addLimitPlane( -joint->getGlobalAxis(), -(NxReal)params->getMinSeparation()*n_axis + ( p + p2 ) / 2  );
   }  
   if( params->haveStopBounce() ) {
-    // TODO
   }
   if( params->haveStopErrorCorrection() ) {
-    // TODO
   }
 
   // sliderForce
@@ -1603,7 +1581,6 @@ void PhysXCallbacks::getSliderJointParameters( void *data ) {
     }
   }
   if( params->haveSeparationRate() ) {
-    // TODO
   }
 }
 
@@ -1895,7 +1872,7 @@ PhysXCallbacks::ConstraintInfo * PhysXCallbacks::attachMotorJoint( ConstraintPar
     static_cast< PhysXSpecificData * >(params->getEngine()->getEngineSpecificData());
   ConstraintInfo * constraint_info = new ConstraintInfo;
   NxActor * actor1 = (NxActor*) params->getBody1();
-  NxActor * actor2 = (NxActor*) static_cast< MotorJointParameters * >(params)->getBody2();;
+  NxActor * actor2 = (NxActor*) static_cast< MotorJointParameters * >(params)->getBody2();
 
   return constraint_info;
 }
