@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -36,16 +36,17 @@
 #include <H3D/Coordinate.h>
 #include <H3D/H3DHapticsDevice.h>
 #include <HAPI/HAPIHapticsDevice.h>
+#include <H3DUtil/H3DTimer.h>
 
 #include <iostream>
 
 using namespace H3D;
-using namespace HAPI;
 
 PhysicsEngineThread::PhysicsEngineThread( const string & _engine,
                                           Priority thread_priority,
                                           int thread_frequency,
-                                          bool _useMainThread ):
+                                          bool _useMainThread,
+                                          bool _use_synchronization ):
   PeriodicThread( thread_priority, thread_frequency ),
   engine( _engine ),
   simulation_running( false ),
@@ -56,8 +57,14 @@ PhysicsEngineThread::PhysicsEngineThread( const string & _engine,
   step_size( (H3DFloat) (1.0f / thread_frequency) ),
   frame_rate_interval ( ((H3DFloat)(1.0f / thread_frequency))*10.0 ),
   world_params ( new WorldParameters ),
-  useMainThread ( _useMainThread ) {
-  
+  useMainThread ( _useMainThread ),
+  synchroniseTheThreads( false ),
+  stepping_simulation( false ),
+  use_synchronization( _use_synchronization ),
+  nr_physics_steps( 1 ),
+  simulation_cb_handle( -1 ),
+  update_cb_handle( -1 ) {
+
   // initialize the physics engine in use
   callbacks()->initEngine( this );  
 }
@@ -167,7 +174,24 @@ void PhysicsEngineThread::synchronousCallback( CallbackFunc func, void *data ) {
     if ( pthread_equal ( getThreadId(), ThreadBase::getCurrentThreadId() ) ) {
       func ( data );
     } else {
-      PeriodicThread::synchronousCallback ( func, data );
+      if( !use_synchronization ) {
+        PeriodicThread::synchronousCallback( func, data );
+      } else {
+        // Schedule the callback after any existing asynchronous callbacks by adding it to callbacks_added
+        // instead of callbacks. The default implementation would execute it first. This ensures that callbacks
+        // to add objects to the simulation will wait for the current asynchronous simulation steps to complete
+        // before modifying the simulation.
+        callback_lock.lock();
+        // signal the thread that a new callback is available if it is waiting for one.
+        if( frequency < 0 && PeriodicThread::callbacks.size() == 0 ) callback_lock.signal();
+        // add the new callback.
+        callbacks_added_lock.lock();  // Have to get added_lock in order to call genCallbackId in a thread safe way
+        callbacks_added.push_back( make_pair( genCallbackId(), make_pair( func, data ) ) );
+        callbacks_added_lock.unlock();
+        // wait for the callback to be done.
+        callback_lock.wait();
+        callback_lock.unlock();
+      }
     }
   }
 }
@@ -210,11 +234,37 @@ void PhysicsEngineThread::setWorldParameters( WorldParameters& wp ) {
   world_params_lock.unlock();
 
   // World parameters must be set before body parameters, so use synchronous callback
-  // todo: Implement better solution if speed is a problem
-  //asynchronousCallback( callbacks()->setWorldParameters, wp );
   synchronousCallback( callbacks()->setWorldParameters, &wp );
 }
 
+void PhysicsEngineThread::synchroniseWithSceneGraph( ) {
+  
+  synchroniseTheThreads_lock.lock();
+  if ( synchroniseTheThreads ){
+    synchronousCallback( callbacks()->synchroniseWithSceneGraph, NULL );
+    synchroniseTheThreads = false;
+  }
+  synchroniseTheThreads_lock.unlock();
+  
+  nodesToDeleteInSync_lock.lock();
+  nodesToDeleteInSync.clear();
+  nodesToDeleteInSync_lock.unlock();
+}
+
+void PhysicsEngineThread::addNodeToDeleteInSynch( Node *n ) {
+  
+  nodesToDeleteInSync_lock.lock();
+  nodesToDeleteInSync.push_back( n );
+  nodesToDeleteInSync_lock.unlock();
+}
+
+void PhysicsEngineThread::synchroniseAtTheEndOfTheFrame() {
+  
+  synchroniseTheThreads_lock.lock();
+  synchroniseTheThreads = true;
+  synchroniseTheThreads_lock.unlock();
+
+}
 
 H3DCollidableId PhysicsEngineThread::addCollidable( CollidableParameters& p ) {
 
@@ -356,46 +406,90 @@ bool PhysicsEngineThread::setRigidBodyParameters( H3DBodyId body_id,
                                                  RigidBodyParameters &p)
 {
 
+  bool have_body = false;
   p.setBodyId( body_id );
   p.setEngine( *this );
+  
   rigid_body_lock.lock();
-  rigid_bodies[body_id]->copyInputParameters( p );
+  RigidBodyMap::iterator i = rigid_bodies.find( body_id );
+  have_body = i != rigid_bodies.end();
+  if( have_body ) { 
+    rigid_bodies[body_id]->copyInputParameters( p );
+  }
   rigid_body_lock.unlock();
 
-  asynchronousCallback( callbacks()->setRigidBodyParameters, &p );
+  if( have_body ) {
+    asynchronousCallback( callbacks()->setRigidBodyParameters, &p );
+  }
+  else {
+    RigidBodyParameters *ptr_p = static_cast< RigidBodyParameters *>( &p );
+    delete ptr_p;
+  }
 
-  return true;
+  return have_body;
 }
 
 bool PhysicsEngineThread::getRigidBodyParameters( H3DBodyId body,
-                                                 RigidBodyParameters& params )
+                                                  RigidBodyParameters& params )
 {
+  bool have_body = false;
   rigid_body_lock.lock();
-  params.copyOutputParameters( *rigid_bodies[body] );
-  params.copyInputParameters( *rigid_bodies[body] );
+  RigidBodyMap::iterator i = rigid_bodies.find( body );
+  have_body = i != rigid_bodies.end();
+  if( have_body ) { 
+    params.copyOutputParameters( *rigid_bodies[body] );
+    params.copyInputParameters( *rigid_bodies[body] );
+  }
   rigid_body_lock.unlock();
 
-  return true;
+  return have_body;
 }
 
 void PhysicsEngineThread::startSimulation() {
   if( !simulation_running ) {
-    // add callback for calculating the rate of the thread and to update
-    // output parameters in objects in the physics engine.
-    simulation_cb_handle = asynchronousCallback( callbacks()->doSimulationSteps, this );
-    save_update_cb_handle = asynchronousCallback( saveUpdateRate, this );
+    if( !use_synchronization ) {
+      // add callback for calculating the rate of the thread and to update
+      // output parameters in objects in the physics engine.
+      simulation_cb_handle = asynchronousCallback( callbacks()->doSimulationSteps, this );
+      update_cb_handle = asynchronousCallback( updateSimulationCB, this );
+    }
     simulation_running = true;
   }
 }
 
 void PhysicsEngineThread::stopSimulation() {
   if( simulation_running ) {
-    removeAsynchronousCallback( simulation_cb_handle );
-    removeAsynchronousCallback( save_update_cb_handle );
+    if( !use_synchronization ) {
+      removeAsynchronousCallback( simulation_cb_handle );
+      removeAsynchronousCallback( update_cb_handle );
+    }
     simulation_running = false;
   }
 }
 
+void PhysicsEngineThread::waitForSimulationSteps() {
+  // If the previous steps are still executing we will wait here
+  H3DTIMER_BEGIN( "waitForSimulationSteps" )
+  stepping_simulation_lock.lock();
+  while( stepping_simulation ) {
+    stepping_simulation_lock.wait();
+  }
+  stepping_simulation_lock.unlock();
+  H3DTIMER_END( "waitForSimulationSteps" )
+}
+
+void PhysicsEngineThread::beginSimulationSteps( std::size_t nr_steps ) {
+  stepping_simulation_lock.lock();
+  stepping_simulation = true;
+  stepping_simulation_lock.unlock();
+  nr_physics_steps = nr_steps;
+  asynchronousCallback( executeSimulationSteps, this );
+}
+
+void PhysicsEngineThread::beginSynchronizeSteps() {
+  asynchronousCallback( updateSimulationCB, this );
+}
+
 int PhysicsEngineThread::setGlobalExternalForce( H3DBodyId body_id,
                                                 const Vec3f &force )
 {
@@ -512,7 +606,8 @@ int PhysicsEngineThread::getCurrentContacts( list< ContactParameters > &contacts
   return 0;
 }
 
-void PhysicsEngineThread::updateObjectParameters() {
+void PhysicsEngineThread::updateRigidBodyParameters() {
+  H3DTIMER_BEGIN( "updateRigidBodyParameters" )
   rigid_body_lock.lock();
   for( RigidBodyMap::iterator i = rigid_bodies.begin();
     i != rigid_bodies.end(); ++i ) {
@@ -526,18 +621,23 @@ void PhysicsEngineThread::updateObjectParameters() {
     callbacks()->getConstraintParameters( (*i).second );
   }
   constraint_lock.unlock();
+  H3DTIMER_END( "updateRigidBodyParameters" )
 }
 
 void PhysicsEngineThread::preUpdateCollidable ( CollidableParameters& p ) {
   if ( ShapeParameters* s= dynamic_cast<ShapeParameters*>(&p) ) {
-    if ( s->getOriginalShape() ) {
+    if ( s->haveOriginalShape() && s->getOriginalShape() ) {
       s->setShape ( static_cast<X3DGeometryNode*>(s->getOriginalShape()->clone()) );
       if ( p.haveScale() ) {
         if ( (p.getScale()-Vec3f ( 1, 1, 1 )).lengthSqr() > H3DUtil::Constants::f_epsilon ) {
           scaleGeometry ( *s->getShape(), p.getScale() );
         }
       }
-      s->getShape()->boundTree->upToDate();
+
+      s->getShape()->setName( "clone_" + s->getOriginalShape()->getName() );
+      if( s->getUpdateShapeBounds() ) {
+        s->getShape()->boundTree->upToDate();
+      }
     }
   }
 }
@@ -585,107 +685,25 @@ void PhysicsEngineThread::scaleGeometry ( X3DGeometryNode& geometry, const Vec3f
   }
 }
 
-/// \todo This function should be renamed.
-H3DUtil::PeriodicThread::CallbackCode PhysicsEngineThread::saveUpdateRate(void *data) {
-
-  PROFILE_BEGIN ( saveUpdateRate );
-
+H3DUtil::PeriodicThread::CallbackCode PhysicsEngineThread::updateSimulationCB(void *data) {
+  H3DTIMER_BEGIN( "updateSimulationCB" )
   PhysicsEngineThread *pt = static_cast< PhysicsEngineThread * >( data );
-  H3DTime current_time = TimeStamp();
-  if( pt->time_of_last_update != 0 ) {
-    H3DTime dt = current_time  - pt->time_of_last_update;
-    if ( dt > pt->frame_rate_interval ) {
-      pt->update_rate = (int) (pt->frame_count / dt);
-      pt->time_of_last_update = current_time;
-      pt->frame_count = 0;
-    }
-  } else {
-    pt->time_of_last_update = current_time;
-  }
-  ++(pt->frame_count);
-  pt->updateObjectParameters();
+  pt->updateSimulation();
+  H3DTIMER_END( "updateSimulationCB" )
 
-  //
-  PhysicsEngineParameters::WorldParameters world_params = 
-    pt->getWorldParameters();
+  return pt->use_synchronization ? 
+    H3DUtil::PeriodicThread::CALLBACK_DONE : 
+    H3DUtil::PeriodicThread::CALLBACK_CONTINUE;
+}
 
-  // only step if RigidBodyCollection is enabled.
-  if( world_params.getEnabled() ) {
+PeriodicThread::CallbackCode PhysicsEngineThread::executeSimulationSteps( void* data ) {
+  PhysicsEngineThread* pt = static_cast<PhysicsEngineThread*>(data);
 
-#ifdef USE_CONTACTS_LOCK
-    pair< ContactList*,
-      PhysicsEngineThread* > data = make_pair( &pt->currentContacts, pt );
-
-    pt->currentContacts_lock.lock();
-    pt->currentContacts.clear();
-    pt->callbacks()->getCurrentContacts ( &data );
-    pt->currentContacts_lock.unlock();
-#endif
-
-    // Get time step to apply damping forces
-    H3DFloat timeStep= pt->getStepSize();
-    if ( pt->getUpdateRate() > 0 ) {
-      timeStep= 1.0f/pt->getUpdateRate();
-    }
-
-    // For backward compatibility
-    // (timeStep not previously considered when adding damping
-    // i.e., timestep=1 when running at exactly desired loop rate) 
-    timeStep/= pt->getStepSize();
-
-    // add damping to each rigid body that has damping enabled
-    list< H3DBodyId > bodies;
-    pt->rigid_body_lock.lock();
-    pt->getCurrentRigidBodies( bodies );
-    pt->rigid_body_lock.unlock();
-
-    for( list< H3DBodyId >::iterator i = bodies.begin(); 
-      i != bodies.end(); ++i ) {
-        PhysicsEngineParameters::RigidBodyParameters p;
-        pt->getRigidBodyParameters( *i, p );
-        PhysicsEngineParameters::ExternalForceTorqueParameters *f_params =
-          &pt->force_torque_params;
-        f_params->engine_thread = pt;
-        f_params->body_id = *i;
-        f_params->position = p.getPosition();
-        if( p.getAutoDamp() ) {  
-          f_params->force = p.getLinearVelocity() * -p.getLinearDampingFactor() * timeStep;
-          f_params->torque = p.getAngularVelocity() * -p.getAngularDampingFactor() * timeStep;
-
-          f_params->force += p.getForce() + p.getGraphicsFrameForce();
-          f_params->torque += p.getTorque() + p.getGraphicsFrameTorque();
-        } else {
-          f_params->force = p.getForce() + p.getGraphicsFrameForce();
-          f_params->torque = p.getTorque() + p.getGraphicsFrameTorque();
-        }
-
-        // Add manipulation forces from haptics devices
-        pt->addHapticInteractionForces ( p, *f_params );
-
-        pt->callbacks()->addGlobalExternalForceAndTorque( f_params );
-
-#ifdef DEBUG_RB_LAG
-      if ( p.getDebug() ) {
-
-        FILE* lagDebugFile= pt->lagDebugFiles[*i];
-
-        if ( !lagDebugFile ) {
-          stringstream ss;
-          ss << "LagDebugPhysics" << *i;
-          lagDebugFile= fopen ( ss.str().c_str(), "w" );
-          pt->lagDebugFiles[*i]= lagDebugFile;
-        }
-        LARGE_INTEGER time;
-        QueryPerformanceCounter ( &time );
-        fprintf ( lagDebugFile, "%lli, %f, %f, %f\n", time.QuadPart, p.getPosition().x, p.getPosition().y, p.getPosition().z );
-      }
-#endif
-    }
+  for( std::size_t i = 0; i < pt->nr_physics_steps; ++i ) {
+    pt->callbacks()->doSimulationSteps( data );
   }
 
-  PROFILE_END ();
-
-  return H3DUtil::PeriodicThread::CALLBACK_CONTINUE;
+  return PeriodicThread::CALLBACK_DONE;
 }
 
 void PhysicsEngineThread::setHapticsDevices ( const NodeVector& _hapticsDevices ) {
@@ -701,45 +719,65 @@ void PhysicsEngineThread::addHapticInteractionForces (
   hapticsDevices_lock.lock();
   // For each haptics device
   for ( NodeVector::const_iterator i= hapticsDevices.begin(); i != hapticsDevices.end(); ++i ) {
-    HAPIHapticsDevice* hd= static_cast<H3DHapticsDevice*>(*i)->getHAPIDevice();
-    // For each layer
-    for( unsigned int layer = 0; layer < hd->nrLayers(); ++layer ) {
-      HAPI::HAPIHapticsRenderer* renderer = 
-        hd->getHapticsRenderer( layer );
-      if( renderer ) {
-        HAPI::HAPIHapticsRenderer::Contacts contacts= renderer->getContacts();
-        // For each contact
-        for( HAPI::HAPIHapticsRenderer::Contacts::iterator j = contacts.begin();
-              j != contacts.end(); ++j ) {
-          // The geometry in contact
-          X3DGeometryNode *geom =
-            static_cast< X3DGeometryNode * >( (*j).first->getUserData() );
-          HAPISurfaceObject::ContactInfo contact= (*j).second;
+    HAPI::HAPIHapticsDevice* hd= static_cast<H3DHapticsDevice*>(*i)->getHAPIDevice();
+    if( hd ) {
+      // For each layer
+      for( unsigned int layer = 0; layer < hd->nrLayers(); ++layer ) {
+        HAPI::HAPIHapticsRenderer* renderer = 
+          hd->getHapticsRenderer( layer );
+        if( renderer ) {
+          HAPI::HAPIHapticsRenderer::Contacts contacts= renderer->getContacts();
+          // For each contact
+          for( HAPI::HAPIHapticsRenderer::Contacts::iterator j = contacts.begin();
+                j != contacts.end(); ++j ) {
+            // The geometry in contact
+            X3DGeometryNode *geom =
+              static_cast< X3DGeometryNode * >( (*j).first->getUserData() );
+            HAPI::HAPISurfaceObject::ContactInfo contact= (*j).second;
 
-          Matrix4d global_to_local= (*j).first->getInverse();
-          Matrix3d global_vec_to_local = global_to_local.getRotationPart();
+            Matrix4d global_to_local= (*j).first->getInverse();
+            Matrix3d global_vec_to_local = global_to_local.getRotationPart();
 
-          Vec3f cp( global_to_local * contact.globalSurfaceContactPoint() );
-          Vec3f f( global_vec_to_local * contact.globalForce() );
+            Vec3f cp( global_to_local * contact.globalSurfaceContactPoint() );
+            Vec3f f( global_vec_to_local * contact.globalForce() );
 
-          // Is the contacted geometry attached to this rigid body?
-          vector< H3DCollidableId > geometries= params.getGeometry();
-          for ( vector< H3DCollidableId >::iterator i= geometries.begin(); i != geometries.end(); ++i ) {
-            collidable_lock.lock();
-            CollidableParameters* c= collidables[ *i ];
-            /// \todo Recurse inside OffsetParameters
-            if ( ShapeParameters* s= dynamic_cast<ShapeParameters*>(c) ) {
-              if ( s->getOriginalShape() == geom ) {
-                // The haptics device is in contact with this rigid body
-                Vec3 global_point= params.getOrientation()*cp + params.getPosition();
-                Vec3 global_force= params.getOrientation()*(-f);
+            // Is the contacted geometry attached to this rigid body?
+            vector< H3DCollidableId > geometries= params.getGeometry();
+            unsigned int actor_index = 0;
+            for ( vector< H3DCollidableId >::iterator k= geometries.begin(); k != geometries.end(); ++k ) {
+              collidable_lock.lock();
+              CollidableParameters* c= collidables[ *k ];
+              if ( ShapeParameters* s= dynamic_cast<ShapeParameters*>(c) ) {
+                if( s->getOriginalShape() == geom ) {
+                  // The haptics device is in contact with this rigid body
+                  Rotation body_orientation;
+                  Vec3f body_position;
+                  if( !params.getPositions().empty() && !params.getOrientations().empty() ) {
+                    const RigidBodyParameters::OrientationList &orn_list = params.getOrientations();
+                    const RigidBodyParameters::PositionList &pos_list = params.getPositions();
+                    if( actor_index < orn_list.size() ) {
+                      body_orientation = orn_list[actor_index];
+                    }
+                    if( actor_index < pos_list.size() ) {
+                      body_position = pos_list[actor_index];
+                    }
+                  } else {
+                    body_orientation = params.getOrientation();
+                    body_position = params.getPosition();
+                  }
 
-                Vec3 lever_arm= global_point - params.getPosition();
-                forceParams.force+= Vec3f(global_force);
-                forceParams.torque+= Vec3f(lever_arm.crossProduct( global_force ));
+                  HAPI::Vec3 global_point= body_orientation*cp + body_position;
+                  HAPI::Vec3 global_force= body_orientation*(-f);
+
+                  HAPI::Vec3 lever_arm= global_point - body_position;
+                  forceParams.force+= Vec3f(global_force);
+                  forceParams.torque+= Vec3f(lever_arm.crossProduct( global_force ));
+                  forceParams.actor_index = actor_index;
+                }
               }
+              collidable_lock.unlock();
+              ++actor_index;
             }
-            collidable_lock.unlock();
           }
         }
       }
@@ -748,6 +786,126 @@ void PhysicsEngineThread::addHapticInteractionForces (
   hapticsDevices_lock.unlock();
 }
 
+void PhysicsEngineThread::updateSimulation() {
+  H3DTime current_time = TimeStamp();
+  if( time_of_last_update != 0 ) {
+    H3DTime dt = current_time - time_of_last_update;
+    if( dt > frame_rate_interval ) {
+      update_rate = (int)((frame_count * nr_physics_steps) / dt);
+      time_of_last_update = current_time;
+      frame_count = 0;
+    }
+  } else {
+    time_of_last_update = current_time;
+  }
+  ++(frame_count);
+  updateRigidBodyParameters();
+
+  //
+  PhysicsEngineParameters::WorldParameters world_params =
+    getWorldParameters();
+
+  // only step if RigidBodyCollection is enabled.
+  if( world_params.getEnabled() ) {
+
+#ifdef USE_CONTACTS_LOCK
+    pair< ContactList*,
+      PhysicsEngineThread* > contact_data = make_pair( &currentContacts, this );
+
+    currentContacts_lock.lock();
+    currentContacts.clear();
+    callbacks()->getCurrentContacts( &contact_data );
+    currentContacts_lock.unlock();
+#endif
+
+    // Get time step to apply damping forces
+    H3DFloat timeStep = getStepSize();
+    if( !world_params.getUseStaticTimeStep() && getUpdateRate() > 0 ) {
+      timeStep = 1.0f / getUpdateRate();
+    }
+
+    // For backward compatibility
+    // (timeStep not previously considered when adding damping
+    // i.e., timestep=1 when running at exactly desired loop rate) 
+    timeStep /= getStepSize();
+
+    // add damping to each rigid body that has damping enabled
+    list< H3DBodyId > bodies;
+    rigid_body_lock.lock();
+    getCurrentRigidBodies( bodies );
+    rigid_body_lock.unlock();
+
+    for( list< H3DBodyId >::iterator i = bodies.begin();
+         i != bodies.end(); ++i ) {
+
+      PhysicsEngineParameters::RigidBodyParameters p;
+      // Since the moment getCurrentRigidBodies() copies rigid_bodies to bodies till now,
+      // in case a body was deleted from the rigid_bodies, a rare random crash was
+      // happening. Return value of getRigidBodyParameters handles that scenario.
+      if( !getRigidBodyParameters( *i, p ) )
+        break;
+      PhysicsEngineParameters::ExternalForceTorqueParameters *f_params =
+        &force_torque_params;
+      f_params->engine_thread = this;
+      f_params->body_id = *i;
+      f_params->position = p.getPosition();
+      if( p.getAutoDamp() ) {
+        f_params->force = p.getLinearVelocity() * -p.getLinearDampingFactor() * timeStep;
+        f_params->torque = p.getAngularVelocity() * -p.getAngularDampingFactor() * timeStep;
+
+        f_params->force += p.getForce() + p.getGraphicsFrameForce();
+        f_params->torque += p.getTorque() + p.getGraphicsFrameTorque();
+      } else {
+        f_params->force = p.getForce() + p.getGraphicsFrameForce();
+        f_params->torque = p.getTorque() + p.getGraphicsFrameTorque();
+      }
+
+      // Add manipulation forces from haptics devices
+      addHapticInteractionForces( p, *f_params );
+
+      callbacks()->addGlobalExternalForceAndTorque( f_params );
+
+      // reset H3D objects forces and torques
+      bool have_body = false;
+      rigid_body_lock.lock();
+      // Since the moment getCurrentRigidBodies() copies rigid_bodies to bodies till now,
+      // in case a body was deleted from the rigid_bodies, a rare random crash was
+      // happening. Check, therefore, for the existance of the body again.
+      RigidBodyMap::iterator bf = rigid_bodies.find( *i );
+      have_body = bf != rigid_bodies.end();
+      if( have_body ) {
+        rigid_bodies[*i]->setForce( Vec3f( 0, 0, 0 ) );
+        rigid_bodies[*i]->setTorque( Vec3f( 0, 0, 0 ) );
+      }
+      rigid_body_lock.unlock();
+
+#ifdef DEBUG_RB_LAG
+      if( p.getDebug() ) {
+
+        FILE* lagDebugFile = lagDebugFiles[*i];
+
+        if( !lagDebugFile ) {
+          stringstream ss;
+          ss << "LagDebugPhysics" << *i;
+          lagDebugFile = fopen( ss.str().c_str(), "w" );
+          lagDebugFiles[*i] = lagDebugFile;
+        }
+        LARGE_INTEGER time;
+        QueryPerformanceCounter( &time );
+        fprintf( lagDebugFile, "%lli, %f, %f, %f\n", time.QuadPart, p.getPosition().x, p.getPosition().y, p.getPosition().z );
+      }
+#endif
+    }
+  }
+
+  if( use_synchronization ) {
+    stepping_simulation_lock.lock();
+    stepping_simulation = false;
+    stepping_simulation_lock.signal();
+    stepping_simulation_lock.unlock();
+  }
+}
+
 bool PhysicsEngineThread::supportsPhysicsEngine( const string &engine ) {
   return registered_physics_engines().find( engine ) != registered_physics_engines().end();
 }
