@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -41,7 +41,11 @@ using namespace H3DUtil;
 
 #ifdef HAVE_PYTHON
 
+#if defined(__APPLE__) && defined(__MACH__) && defined( HAVE_PYTHON_OSX_FRAMEWORK )
+#include <Python/Python.h>
+#else
 #include <Python.h>
+#endif
 
 namespace PyFunctions {
   int H3DPyDouble_Check( PyObject *v ) {
@@ -258,9 +262,9 @@ PyObject* TrackedMField_eraseTracked( PyObject *self, PyObject *args );
 
 namespace {
   static PyMethodDef H3DPhysics_PythonMethods[] = {
-      {"TrackedMField_insertTracked", TrackedMField_insertTracked, 0},
-      {"TrackedMField_updateTracked", TrackedMField_updateTracked, 0},
-      {"TrackedMField_eraseTracked",  TrackedMField_eraseTracked, 0},
+      {"TrackedMField_insertTracked", TrackedMField_insertTracked, METH_VARARGS, NULL},
+      {"TrackedMField_updateTracked", TrackedMField_updateTracked,  METH_VARARGS, NULL},
+      {"TrackedMField_eraseTracked",  TrackedMField_eraseTracked,  METH_VARARGS, NULL},
       {NULL, NULL, 0, NULL}        /* Sentinel */
   };
 }
@@ -281,9 +285,6 @@ void H3DPhysicsPythonInterface::initModule () {
   // Py_Initialize really should be done in the DLL loader function:
   if ( !Py_IsInitialized() ) {
     Py_Initialize();  
-    /// \todo Transfer command line args
-    //if( argv )
-    //  PySys_SetArgv(argc,argv);
     PythonScript::disallowMainThreadPython();
   }
 
@@ -291,12 +292,31 @@ void H3DPhysicsPythonInterface::initModule () {
   PyGILState_STATE state = PyGILState_Ensure();
 
   // Standard H3D Python modules must be created first
-  PythonInternals::initH3D();
+  PythonInternals::initH3DInternal();
 
   // Create H3DPhysics module and functions
+#if PY_MAJOR_VERSION >= 3
+  static struct PyModuleDef moduledef = {
+    PyModuleDef_HEAD_INIT,
+    "H3DPhysics",     /* m_name */
+    "This is a module for H3DPhysics",  /* m_doc */
+    -1,                  /* m_size */
+    H3DPhysics_PythonMethods,    /* m_methods */
+    NULL,                /* m_reload */
+    NULL,                /* m_traverse */
+    NULL,                /* m_clear */
+    NULL,                /* m_free */
+  };
+
+  PyObject* h3dPhysicsModule =  PyModule_Create(&moduledef);
+#else
   PyObject* h3dPhysicsModule= Py_InitModule("H3DPhysics", H3DPhysics_PythonMethods);
+#endif
   PyObject* h3dPhysicsModuleDict= PyModule_GetDict( h3dPhysicsModule );
 
+  
+  PyImport_AddModule( "H3DPhysics" );
+
   // Add builtins to H3DPhysics module
   if (PyDict_GetItemString( static_cast< PyObject * >(h3dPhysicsModuleDict), "__builtins__") == NULL) {
     if (PyDict_SetItemString( static_cast< PyObject * >(h3dPhysicsModuleDict), "__builtins__",
@@ -335,7 +355,7 @@ PyObject* TrackedMField_insertTracked( PyObject *self, PyObject *args ) {
 
   // Get field argument
   PyObject *field = PyTuple_GetItem( args, 0 );
-  if( ! PyInstance_Check( field ) ) {
+  if( !PyObject_TypeCheck( field, &PyBaseObject_Type ) ) {
     PyErr_SetString( PyExc_ValueError, 
 "Invalid Field type given as argument to H3DPhysics.TrackedMField_insertTracked( self, index, value )" );
     return 0;
@@ -373,7 +393,7 @@ call the base class __init__ function." );
     return 0;
   }
   Field *field_ptr = static_cast< Field * >
-    ( PyCObject_AsVoidPtr( py_field_ptr ) );
+    ( PyCapsule_GetPointer( py_field_ptr, NULL ) );
 
   if( field_ptr && !indices.empty() ) {
     // Get value argument
@@ -406,7 +426,7 @@ PyObject* TrackedMField_updateTracked( PyObject *self, PyObject *args ) {
 
   // Get field argument
   PyObject *field = PyTuple_GetItem( args, 0 );
-  if( ! PyInstance_Check( field ) ) {
+  if( !PyObject_TypeCheck( field, &PyBaseObject_Type ) ) {
     PyErr_SetString( PyExc_ValueError, 
 "Invalid Field type given as argument to H3DPhysics.TrackedMField_updateTracked( self, index, value )" );
     return 0;
@@ -444,7 +464,7 @@ call the base class __init__ function." );
     return 0;
   }
   Field *field_ptr = static_cast< Field * >
-    ( PyCObject_AsVoidPtr( py_field_ptr ) );
+    ( PyCapsule_GetPointer( py_field_ptr, NULL ) );
 
 
   if( field_ptr && !indices.empty() ) {
@@ -478,7 +498,7 @@ PyObject* TrackedMField_eraseTracked( PyObject *self, PyObject *args ) {
 
   // Get field argument
   PyObject *field = PyTuple_GetItem( args, 0 );
-  if( ! PyInstance_Check( field ) ) {
+  if( !PyObject_TypeCheck( field, &PyBaseObject_Type ) ) {
     PyErr_SetString( PyExc_ValueError, 
 "Invalid Field type given as argument to H3DPhysics.TrackedMField_eraseTracked( self, index, count= 1 )" );
     return 0;
@@ -528,7 +548,7 @@ call the base class __init__ function." );
     return 0;
   }
   Field *field_ptr = static_cast< Field * >
-    ( PyCObject_AsVoidPtr( py_field_ptr ) );
+    ( PyCapsule_GetPointer( py_field_ptr, NULL ) );
 
   if( field_ptr && !indices.empty() ) {
     bool success;
