@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -17,7 +17,7 @@
 //    along with H3D API; if not, write to the Free Software
 //    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
-//    A commercial license is also available. Please contact us at 
+//    A commercial license is also available. Please contact us at
 //    www.sensegraphics.com for more information.
 //
 //
@@ -34,39 +34,39 @@
 
 using namespace H3D;
 
-H3DNodeDatabase RigidBody::database( "RigidBody", 
-                                    &(newInstance< RigidBody >), 
+H3DNodeDatabase RigidBody::database( "RigidBody",
+                                    &(newInstance< RigidBody >),
                                     typeid( RigidBody ),
                                     &H3DBodyNode::database);
 
 namespace RigidBodyInternals {
-  FIELDDB_ELEMENT( RigidBody, angularDampingFactor, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBody, angularVelocity     , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBody, autoDamp            , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBody, autoDisable         , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBody, centerOfMass        , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBody, disableAngularSpeed , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBody, disableLinearSpeed  , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBody, disableTime         , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBody, enabled             , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBody, finiteRotationAxis  , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBody, fixed               , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBody, forces              , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBody, geometry            , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBody, inertia              , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBody, linearDampingFactor , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBody, linearVelocity      , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBody, mass                , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBody, massDensityModel    , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBody, orientation         , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBody, position            , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBody, torques             , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBody, useFiniteRotation   , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBody, useGlobalGravity    , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBody, engineOptions       , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBody, kinematicControl    , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBody, debug               , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBody, transform           , INPUT_OUTPUT );
+  FIELDDB_ELEMENT( RigidBody, angularDampingFactor, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBody, angularVelocity     , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBody, autoDamp            , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBody, autoDisable         , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBody, centerOfMass        , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBody, disableAngularSpeed , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBody, disableLinearSpeed  , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBody, disableTime         , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBody, enabled             , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBody, finiteRotationAxis  , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBody, fixed               , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBody, forces              , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBody, geometry            , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBody, inertia             , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBody, linearDampingFactor , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBody, linearVelocity      , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBody, mass                , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBody, massDensityModel    , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBody, orientation         , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBody, position            , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBody, torques             , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBody, useFiniteRotation   , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBody, useGlobalGravity    , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBody, engineOptions       , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBody, kinematicControl    , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBody, debug               , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBody, transform           , INPUT_OUTPUT )
 }
 
 RigidBody::RigidBody(Inst< SFNode >  _metadata,
@@ -125,29 +125,30 @@ kinematicControl ( _kinematicControl ),
 valueUpdater( _valueUpdater ),
 engineOptions ( _engineOptions ),
 transform( _transform ),
-debug ( new SFBool ) {
+debug ( new SFBool ),
+nbframes_collidables_without_ids( 0 ) {
 
   // init fields
   type_name = "RigidBody";
   database.initFields( this );
 
-  valueUpdater->setName( "ValueUpdater" );
+  valueUpdater->setName( "valueUpdater" );
   valueUpdater->setOwner( this );
 
   // set default values
   angularDampingFactor->setValue( (H3DFloat)0.001 );
-  angularVelocity->setValue( Vec3f( 0, 0, 0 ) ); 
+  angularVelocity->setValue( Vec3f( 0, 0, 0 ) );
   autoDamp->setValue( false );
-  autoDisable->setValue( false );  
+  autoDisable->setValue( false );
   centerOfMass->setValue( Vec3f( 0, 0, 0 ) );
   disableAngularSpeed->setValue( 0 );
   disableLinearSpeed->setValue( 0 );
   disableTime->setValue( 0 );
-  enabled->setValue( true );  
+  enabled->setValue( true );
   finiteRotationAxis->setValue( Vec3f( 0, 0, 0 ) );
   fixed->setValue( false );
   inertia->setValue( Matrix3f() );
-  linearDampingFactor->setValue( (H3DFloat)0.001 );  
+  linearDampingFactor->setValue( (H3DFloat)0.001 );
   linearVelocity->setValue( Vec3f( 0, 0, 0 ) );
   mass->setValue( 1 );
   orientation->setValue( Rotation( 0, 0, 1, 0 ) );
@@ -197,7 +198,7 @@ void RigidBody::traverseSG(H3D::TraverseInfo &ti) {
 
         position->setValue(t*position->getValue());
         orientation->setValue(Rotation(t.getRotationPart())*orientation->getValue());
-      }
+        }
       initializeBody( *pt );
     }
 
@@ -205,7 +206,7 @@ void RigidBody::traverseSG(H3D::TraverseInfo &ti) {
       // traverse the collidables
       for( MFCollidableNode::const_iterator i = geometry->begin();
         i != geometry->end(); ++i ) {
-          X3DNBodyCollidableNode *col = 
+          X3DNBodyCollidableNode *col =
             static_cast< X3DNBodyCollidableNode * >(*i);
           if( col ) col->traverseSG( ti );
       }
@@ -215,7 +216,7 @@ void RigidBody::traverseSG(H3D::TraverseInfo &ti) {
 
       // NF. Because there are fields that are both input and output,
       // like position and orientation (to support kinematic objects in Bullet and similar),
-      // we need to ensure that any user-defined values are set before they are overwritten 
+      // we need to ensure that any user-defined values are set before they are overwritten
       // with current values retrieved from the simulation
       // NB. This is asyncronous, so the values may still be overwritten temporarily, but they will sync in the end.
       valueUpdater->upToDate();
@@ -228,14 +229,21 @@ void RigidBody::traverseSG(H3D::TraverseInfo &ti) {
       orientation->unroute( valueUpdater );
       linearVelocity->unroute( valueUpdater );
       angularVelocity->unroute( valueUpdater );
+      enabled->unroute( valueUpdater );
 
       // update output fields
       PhysicsEngineParameters::RigidBodyParameters params;
       pt->getRigidBodyParameters( getBodyId(), params );
-      position->setValue( params.getPosition() );
-      orientation->setValue( params.getOrientation() );
+      Vec3f pos = params.getPosition();
+      Rotation orn = params.getOrientation();
+      if( pos != position->getValue() )
+          position->setValue(pos);
+      if( orn != orientation->getValue() )
+        orientation->setValue( orn );
+
       linearVelocity->setValue( params.getLinearVelocity() );
       angularVelocity->setValue( params.getAngularVelocity() );
+      if( params.getEnabled() != enabled->getValue() ) enabled->setValue( params.getEnabled() );
 
       // restore routes
       position->routeNoEvent( valueUpdater );
@@ -243,6 +251,26 @@ void RigidBody::traverseSG(H3D::TraverseInfo &ti) {
       linearVelocity->routeNoEvent( valueUpdater );
       angularVelocity->routeNoEvent( valueUpdater );
 
+      if( !collidables_without_ids.empty() ) {
+          // update collidables that are not yet updated
+          geometry->touch();
+          nbframes_collidables_without_ids++;
+
+          if( nbframes_collidables_without_ids > 10 ) {
+            //
+            H3D::Console( 4 ) << " The following collidables did not get valid ids. Hence, not been added to the underlying physics body" << std::endl;
+            for( unsigned int idx = 0; idx < collidables_without_ids.size(); idx++ ) {
+              std::pair<unsigned int, std::string> val = collidables_without_ids[idx];
+              H3D::Console( 4 ) << "Name: "<< val.second << "\t index: " << val.first << std::endl;
+            }
+            nbframes_collidables_without_ids = 0;
+            collidables_without_ids.clear();
+          }
+      }
+      else {
+        nbframes_collidables_without_ids = 0;
+      }
+
 #ifdef DEBUG_RB_LAG
     if ( debug->getValue() ) {
       if ( !lagDebugFile.is_open() ) {
@@ -259,9 +287,9 @@ void RigidBody::traverseSG(H3D::TraverseInfo &ti) {
   }
 }
 
-void RigidBody::applyForces( PhysicsEngineThread *pt) {  
-  const vector< Vec3f > &force_vector = forces->getValue();  
-  const vector< Vec3f > &torque_vector = torques->getValue();  
+void RigidBody::applyForces( PhysicsEngineThread *pt) {
+  const vector< Vec3f > &force_vector = forces->getValue();
+  const vector< Vec3f > &torque_vector = torques->getValue();
 
   Vec3f total_force, total_torque;
 
@@ -310,7 +338,7 @@ void RigidBody::applyForces( PhysicsEngineThread *pt) {
             Vec3f global_force = orn * (-force[i]);
             Vec3f lever_arm = global_point - pos;
             total_force = total_force +  global_force;
-            total_torque = total_torque + lever_arm.crossProduct( global_force ); 
+            total_torque = total_torque + lever_arm.crossProduct( global_force );
           }
         }
       }
@@ -348,13 +376,23 @@ void RigidBody::ValueUpdater::update() {
   RigidBody *body = static_cast< RigidBody * >( getOwner());
   if ( body->isInitialized() ) {
     RigidBodyParameters *params = body->getRigidBodyParameters();
-    body->engine_thread->setRigidBodyParameters( body->body_id, *params );
+    if( params )
+      body->engine_thread->setRigidBodyParameters( body->body_id, *params );
   }
   EventCollectingField< PeriodicUpdate< Field > >::update();
 }
 
+PhysicsEngineParameters::RigidBodyParameters* RigidBody::createRigidBodyParameters () {
+  return new PhysicsEngineParameters::RigidBodyParameters();
+}
+
 PhysicsEngineParameters::RigidBodyParameters *RigidBody::getRigidBodyParameters( bool all_params ) {
-  PhysicsEngineParameters::RigidBodyParameters *params = new PhysicsEngineParameters::RigidBodyParameters;
+  //PhysicsEngineParameters::RigidBodyParameters *params = new PhysicsEngineParameters::RigidBodyParameters;
+  PhysicsEngineParameters::RigidBodyParameters *params = createRigidBodyParameters();
+
+#ifdef H3DPHYSICS_ENGINE_PARAMETERS_INCLUDE_NAME
+  params->setName( this->getName() );
+#endif
 
   if( all_params || valueUpdater->hasCausedEvent( angularDampingFactor ) ) {
     params->setAngularDampingFactor( angularDampingFactor->getValue() );
@@ -391,12 +429,14 @@ PhysicsEngineParameters::RigidBodyParameters *RigidBody::getRigidBodyParameters(
   }
   if( all_params || valueUpdater->hasCausedEvent( geometry ) ) {
     vector< H3DCollidableId > geom_ids;
-    for( RigidBody::MFCollidableNode::const_iterator i = geometry->begin();
-      i != geometry->end(); ++i ) {
-        X3DNBodyCollidableNode *cs = static_cast< X3DNBodyCollidableNode * >( *i );
-        PhysicsEngineParameters::H3DCollidableId id = cs->getCollidableId();
-        if( id )
-          geom_ids.push_back( id );
+    collidables_without_ids.clear();
+    for( unsigned int i = 0; i < geometry->size(); ++i ) {
+        X3DNBodyCollidableNode *cs = static_cast< X3DNBodyCollidableNode * >( geometry->getValueByIndex(i) );
+        PhysicsEngineParameters::H3DCollidableId collidable_id = cs->getCollidableId();
+        if( collidable_id )
+          geom_ids.push_back( collidable_id );
+        else
+          collidables_without_ids.push_back( std::pair<unsigned int, std::string>(i,cs->getName()) );
     }
     params->setGeometry( geom_ids );
   }
@@ -457,8 +497,24 @@ PhysicsEngineParameters::RigidBodyParameters *RigidBody::getRigidBodyParameters(
   }
   if ( H3DEngineOptions* options= engineOptions->getOptions ( engine_thread->getEngine() ) ) {
     if ( all_params || valueUpdater->hasCausedEvent ( options->valueUpdater ) ) {
-      params->setEngineOptions ( options->valueUpdater->getParameters() );
+      params->setEngineOptions ( options->valueUpdater->getParameters( all_params ) );
     }
   }
   return params;
 }
+void RigidBody::renderCollidable( bool render_only_enabled_collidables ){
+  glMatrixMode( GL_MODELVIEW );
+  glPushMatrix();
+  const Rotation &body_orn = orientation->getValue();
+  const Vec3f &body_pos = position->getValue();
+  glTranslatef( body_pos.x, body_pos.y, body_pos.z );
+  glRotatef( (GLfloat) (body_orn.angle*180/H3DUtil::Constants::pi), body_orn.axis.x, body_orn.axis.y, body_orn.axis.z );
+  for( RigidBody::MFCollidableNode::const_iterator c = geometry->begin(); c != geometry->end(); c++ ) {
+    X3DNBodyCollidableNode *collidable = static_cast< X3DNBodyCollidableNode * >( *c );
+    if( !render_only_enabled_collidables || collidable->enabled->getValue() ) {
+      collidable->displayList->callList();
+    }
+  }
+  glMatrixMode( GL_MODELVIEW );
+  glPopMatrix();
+}
