@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -43,28 +43,33 @@ H3DTime RigidBodyCollection::profileInterval= 5.0;
 #endif
 
 namespace RigidBodyCollectionInternals {
-  FIELDDB_ELEMENT( RigidBodyCollection, set_contacts           , INPUT_ONLY );
-  FIELDDB_ELEMENT( RigidBodyCollection, autoDisable            , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBodyCollection, constantForceMix       , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBodyCollection, contactSurfaceThickness, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBodyCollection, disableAngularSpeed    , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBodyCollection, disableLinearSpeed     , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBodyCollection, disableTime            , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBodyCollection, enabled                , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBodyCollection, errorCorrection        , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBodyCollection, gravity                , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBodyCollection, iterations             , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBodyCollection, maxCorrectionSpeed     , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBodyCollection, preferAccuracy         , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBodyCollection, collider               , INITIALIZE_ONLY );
-  FIELDDB_ELEMENT( RigidBodyCollection, physicsEngine          , INITIALIZE_ONLY );
-  FIELDDB_ELEMENT( RigidBodyCollection, desiredUpdateRate      , INITIALIZE_ONLY );
-  FIELDDB_ELEMENT( RigidBodyCollection, updateRate             , OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RigidBodyCollection, stepUpdateTime         , OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RigidBodyCollection, useMainThread          , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBodyCollection, engineOptions          , INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBodyCollection, bodies, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( RigidBodyCollection, joints, INPUT_OUTPUT );  
+  FIELDDB_ELEMENT( RigidBodyCollection, set_contacts           , INPUT_ONLY )
+  FIELDDB_ELEMENT( RigidBodyCollection, autoDisable            , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBodyCollection, constantForceMix       , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBodyCollection, contactSurfaceThickness, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBodyCollection, disableAngularSpeed    , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBodyCollection, disableLinearSpeed     , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBodyCollection, disableTime            , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBodyCollection, enabled                , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBodyCollection, errorCorrection        , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBodyCollection, gravity                , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBodyCollection, iterations             , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBodyCollection, maxCorrectionSpeed     , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBodyCollection, preferAccuracy         , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBodyCollection, collider               , INITIALIZE_ONLY )
+  FIELDDB_ELEMENT( RigidBodyCollection, physicsEngine          , INITIALIZE_ONLY )
+  FIELDDB_ELEMENT( RigidBodyCollection, desiredUpdateRate      , INITIALIZE_ONLY )
+  FIELDDB_ELEMENT( RigidBodyCollection, updateRate             , OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RigidBodyCollection, stepUpdateTime         , OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RigidBodyCollection, useMainThread          , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBodyCollection, engineOptions          , INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBodyCollection, bodies, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBodyCollection, joints, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBodyCollection, renderCollidables, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBodyCollection, renderOnlyEnabledCollidables, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBodyCollection, useStaticTimeStep, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( RigidBodyCollection, syncGraphicsFrames, INITIALIZE_ONLY )
+  FIELDDB_ELEMENT( RigidBodyCollection, syncPhysicsFrames, INITIALIZE_ONLY )
 }
 
 RigidBodyCollection::RigidBodyCollection( Inst< SFNode  > _metadata,
@@ -91,8 +96,15 @@ RigidBodyCollection::RigidBodyCollection( Inst< SFNode  > _metadata,
                                           Inst< SFInt32 > _updateRate,
                                           Inst< SFTime  > _stepUpdateTime,
                                           Inst< EnableUseMainThread > _useMainThread,
-                                          Inst< MFEngineOptions > _engineOptions ) :
+                                          Inst< MFEngineOptions > _engineOptions,
+                                          Inst< DisplayList > _displayList,
+                                          Inst< SFBool > _renderCollidables,
+                                          Inst< SFBool > _renderOnlyEnabledCollidables,
+                                          Inst< SFBool  > _useStaticTimeStep,
+                                          Inst< SFInt32 > _syncGraphicsFrames,
+                                          Inst< SFInt32 > _syncPhysicsFrames ) :
   X3DChildNode( _metadata ),
+  H3DDisplayListObject( _displayList ),
   set_contacts( _set_contacts ),
   autoDisable( _autoDisable ),
   bodies( _bodies ),
@@ -116,12 +128,21 @@ RigidBodyCollection::RigidBodyCollection( Inst< SFNode  > _metadata,
   updateRate( _updateRate ),
   stepUpdateTime( _stepUpdateTime ),
   useMainThread ( _useMainThread ),
-  engineOptions ( _engineOptions ) {
+  engineOptions ( _engineOptions ),
+  renderCollidables( _renderCollidables ),
+  renderOnlyEnabledCollidables( _renderOnlyEnabledCollidables ), 
+  useStaticTimeStep( _useStaticTimeStep ),
+  syncGraphicsFrames( _syncGraphicsFrames ),
+  syncPhysicsFrames( _syncPhysicsFrames ),
+  nr_graphics_steps( 0 ) {
 
   // initialize fields
   type_name = "RigidBodyCollection";
   database.initFields( this );
 
+  displayList->setOwner( this );
+  displayList->setCacheMode( H3DDisplayListObject::DisplayList::OFF );
+
   // Initialize the H3DPhysics python module
   H3DPhysicsPythonInterface::getInstance();
 
@@ -131,14 +152,17 @@ RigidBodyCollection::RigidBodyCollection( Inst< SFNode  > _metadata,
   enableDisable->setName( "enableDisable" );
   enableDisable->setOwner( this );
 
+  collider->setOwner( this );
+
   // set default values
-  autoDisable->setValue( false );  
+  renderCollidables->setValue( false );
+  autoDisable->setValue( false );
   constantForceMix->setValue( (H3DFloat) 0.00001 );
   contactSurfaceThickness->setValue( 0 );
   disableAngularSpeed->setValue( 0 );
   disableLinearSpeed->setValue( 0 );
   disableTime->setValue( 0 );
-  enabled->setValue( true );  
+  enabled->setValue( true );
   errorCorrection->setValue( 0.2f );
   gravity->setValue( Vec3f(0, -9.8f, 0) );
   iterations->setValue( 10 );
@@ -156,6 +180,10 @@ RigidBodyCollection::RigidBodyCollection( Inst< SFNode  > _metadata,
   updateRate->setValue( 0, id );
   stepUpdateTime->setValue( 0, id );
   useMainThread->setValue ( false );
+  renderOnlyEnabledCollidables->setValue( false );
+  useStaticTimeStep->setValue( false );
+  syncGraphicsFrames->setValue( 0 );
+  syncPhysicsFrames->setValue( 0 );
 
   // set up routes
   autoDisable->route( valueUpdater );
@@ -169,6 +197,7 @@ RigidBodyCollection::RigidBodyCollection( Inst< SFNode  > _metadata,
   iterations->route( valueUpdater );
   maxCorrectionSpeed->route( valueUpdater );
   preferAccuracy->route( valueUpdater );
+  useStaticTimeStep->route( valueUpdater );
   engineOptions->route ( valueUpdater );
 }
 
@@ -181,17 +210,32 @@ RigidBodyCollection::~RigidBodyCollection() {
   }
 }
 
+void RigidBodyCollection::render() {
+  if( renderCollidables->getValue() ) {
+    const bool &render_only_enabled_collidables = renderOnlyEnabledCollidables->getValue();
+    for( MFRigidBody::const_iterator i = bodies->begin(); i != bodies->end(); i++ ) {
+      RigidBody *body = static_cast< RigidBody * >( *i );
+      body->renderCollidable( render_only_enabled_collidables );
+    }
+  }
+}
+
 void RigidBodyCollection::initialize() {
   X3DChildNode::initialize();
 
   const string &engine = physicsEngine->getValue();
 
   if( PhysicsEngineThread::supportsPhysicsEngine( engine ) ) {
+    bool use_synchronization = 
+      syncGraphicsFrames->getValue() > 0 && 
+      syncPhysicsFrames->getValue() > 0;
+
     simulationThread.reset( 
              new PhysicsEngineThread( engine, 
                                       PeriodicThreadBase::NORMAL_PRIORITY, 
                                       desiredUpdateRate->getValue(),
-                                      useMainThread->getValue() ) );
+                                      useMainThread->getValue(),
+                                      use_synchronization ) );
     simulationThread->setThreadName( "RigidBodyCollection " + engine + " engine thread" );
   } else {
     Console( 4 ) << "Warning: Unsupported physics engine "
@@ -202,7 +246,7 @@ void RigidBodyCollection::initialize() {
     if( physics_engine_names.size() > 0 ) {
       Console(4) << ". This build of H3DPhysics supports the following "
                  << "physics engines for RigidBodyCollection:" << endl;
-      int j = 0;
+      size_t j = 0;
       for( set< string >::iterator i = physics_engine_names.begin();
            i != physics_engine_names.end(); ++i, ++j ) {
         Console(4) << "\"" << *i << "\"";
@@ -246,31 +290,14 @@ void RigidBodyCollection::traverseSG(TraverseInfo &ti) {
 #endif
 
   if( simulationThread.get() ) {
-
-    // update list of current devices to use to apply interaction force to bodies
-    if ( DeviceInfo* di= DeviceInfo::getActive() ) {
-      simulationThread->setHapticsDevices ( di->device->getValue() );
+    if( waitForSimulationSteps() ) {
+      beginSimulationSteps();
+      H3DTIMER_BEGIN( "traverseSimulation" )
+      traverseSimulation( ti );
+      H3DTIMER_END( "traverseSimulation" )
+      beginSynchronizeSteps();
     }
 
-    valueUpdater->upToDate ();
-
-    // traverse collider
-    CollisionCollection *cc = collider->getValue();
-    if( cc ) cc->traverseSG( ti );
-
-    // traverse all bodies
-    const NodeVector &c = bodies->getValue();
-    for( unsigned int i = 0; i < c.size(); ++i ) {
-      if( c[i] ) c[i]->traverseSG( ti );
-    }
-
-    // traverse all joints
-    const NodeVector &j = joints->getValue();
-    for( unsigned int i = 0; i < j.size(); ++i ) {
-      if( j[i] ) j[i]->traverseSG( ti );
-    }
-
-
     // update output field values
     updateRate->setValue( simulationThread->getUpdateRate(), id );
     stepUpdateTime->setValue( simulationThread->getLastLoopTime(), id );
@@ -284,6 +311,59 @@ void RigidBodyCollection::traverseSG(TraverseInfo &ti) {
     // Otherwise gravity etc will cause bodies to move before joints have
     // been added, for example.
     enabled->route( enableDisable );
+
+    // Synchronizes with physics engine thread, i.e deletes pending objects.
+    simulationThread->synchroniseWithSceneGraph();
+  }
+}
+
+void H3D::RigidBodyCollection::traverseSimulation( TraverseInfo & ti ) {
+  // update list of current devices to use to apply interaction force to bodies
+  if( DeviceInfo* di = DeviceInfo::getActive() ) {
+    simulationThread->setHapticsDevices( di->device->getValue() );
+  }
+
+  valueUpdater->upToDate();
+
+  // traverse collider
+  CollisionCollection *cc = collider->getValue();
+  if( cc ) cc->traverseSG( ti );
+
+  // traverse all bodies
+  const NodeVector &c = bodies->getValue();
+  for( unsigned int i = 0; i < c.size(); ++i ) {
+    if( c[i] ) c[i]->traverseSG( ti );
+  }
+
+  // traverse all joints
+  const NodeVector &j = joints->getValue();
+  for( unsigned int i = 0; i < j.size(); ++i ) {
+    if( j[i] ) j[i]->traverseSG( ti );
+  }
+}
+
+bool RigidBodyCollection::waitForSimulationSteps() {
+  if( simulationThread->useSynchronization() ) {
+    if( (int)nr_graphics_steps >= syncGraphicsFrames->getValue() ) {
+      simulationThread->waitForSimulationSteps();
+      nr_graphics_steps = 0;
+    }
+    ++nr_graphics_steps;
+    return nr_graphics_steps == 1;
+  } else {
+    return true;
+  }
+}
+
+void RigidBodyCollection::beginSimulationSteps() {
+  if( simulationThread->useSynchronization() ) {
+    simulationThread->beginSimulationSteps( syncPhysicsFrames->getValue() );
+  }
+}
+
+void RigidBodyCollection::beginSynchronizeSteps() {
+  if( simulationThread->useSynchronization() ) {
+    simulationThread->beginSynchronizeSteps();
   }
 }
 
@@ -324,9 +404,12 @@ PhysicsEngineParameters::WorldParameters* RigidBodyCollection::getWorldParameter
   if( all_params || valueUpdater->hasCausedEvent( preferAccuracy ) ) {
     params->setPreferAccuracy( preferAccuracy->getValue() );
   }
+  if( all_params || valueUpdater->hasCausedEvent( useStaticTimeStep ) ) {
+    params->setUseStaticTimeStep( useStaticTimeStep->getValue() );
+  }
   if ( H3DEngineOptions* options= engineOptions->getOptions( simulationThread->getEngine() ) ) {
     if ( all_params || valueUpdater->hasCausedEvent ( options->valueUpdater ) ) {
-      params->setEngineOptions ( options->valueUpdater->getParameters() );
+      params->setEngineOptions ( options->valueUpdater->getParameters( all_params ) );
     }
   }
 
@@ -349,6 +432,7 @@ void RigidBodyCollection::EnableDisable::update() {
 
   if ( pbc->simulationThread.get() ) {
     if ( value ) {
+      pbc->nr_graphics_steps = pbc->syncGraphicsFrames->getValue();
       pbc->simulationThread->startSimulation();
     } else {
       pbc->simulationThread->stopSimulation();
