@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -109,7 +109,6 @@ PeriodicThread::CallbackCode SOFACallbacks::deInitEngine( void *data ) {
   SOFACallbacks::SOFASpecificData *sofa_data = 
     static_cast< SOFACallbacks::SOFASpecificData *>( pt->getEngineSpecificData() );
 
-  // TODO: cleanup
   if (sofa_data->scene_root!=NULL)
     sofa::simulation::getSimulation()->unload(sofa_data->scene_root);
 #ifdef SOFA_DEBUG_PRINTOUTS
@@ -151,6 +150,9 @@ PeriodicThread::CallbackCode SOFACallbacks::doSimulationSteps(void *data) {
   return PeriodicThread::CALLBACK_CONTINUE;
 }
 
+PeriodicThread::CallbackCode SOFACallbacks::synchroniseWithSceneGraph(void *data) {
+  return PeriodicThread::CALLBACK_DONE;
+}
 
 PeriodicThread::CallbackCode SOFACallbacks::setWorldParameters(void *data) {
   PhysicsEngineParameters::WorldParameters *params = static_cast< PhysicsEngineParameters::WorldParameters *>( data );
@@ -160,7 +162,7 @@ PeriodicThread::CallbackCode SOFACallbacks::setWorldParameters(void *data) {
   if( params->haveAutoDisable() || 
     params->haveDisableLinearSpeed() || 
     params->haveDisableAngularSpeed() ){
-      ///TODO adapt to SOFA
+      /// Adapt to SOFA
       //list< H3DBodyId > bodies;
       //params->engine_thread->getCurrentRigidBodies( bodies );
       //for( list< H3DBodyId >::iterator i = bodies.begin(); 
@@ -265,7 +267,6 @@ H3DUtil::PeriodicThread::CallbackCode SOFACallbacks::addRigidBody(void *data) {
   sofa_data->scene_root->setAnimate(true);
   sofa_data->simulationPtr->updateContext(sofa_data->scene_root.get());
 
-  ///todo remove following code line?
   // Add rigid body to simulation
   //sofa_data->scene_root->addChild ( rb->getNode() );
 
@@ -321,7 +322,6 @@ H3DUtil::PeriodicThread::CallbackCode SOFACallbacks::addJoint(void *data) {
   SOFASpecificData *sofa_data = 
     static_cast< SOFASpecificData * >(params->getEngine()->getEngineSpecificData());
 
-  ///TODO add implementation later?!
   Console ( 3 ) << "Warning: Joint type " << params->getType() << " not implement in SOFACallback!" << endl;
 
   return PeriodicThread::CALLBACK_DONE;
@@ -385,8 +385,6 @@ H3DUtil::PeriodicThread::CallbackCode SOFACallbacks::getJointParameters(void *da
   PhysicsEngineParameters::JointParameters *params = 
     static_cast< PhysicsEngineParameters::JointParameters *>( data );
 
-  ///TODO add implementation later...
-
   return PeriodicThread::CALLBACK_DONE;
 }
 
@@ -395,8 +393,6 @@ H3DUtil::PeriodicThread::CallbackCode SOFACallbacks::setJointParameters(void *da
   PhysicsEngineParameters::JointParameters *params = 
     static_cast< PhysicsEngineParameters::JointParameters *>( data );
 
-  ///TODO add implementation later...
-
   return PeriodicThread::CALLBACK_DONE;
 }
 
@@ -404,7 +400,6 @@ H3DUtil::PeriodicThread::CallbackCode SOFACallbacks::setJointParameters(void *da
 PeriodicThread::CallbackCode SOFACallbacks::removeCollidable( void *data ) {
   CollidableParameters *params = static_cast< CollidableParameters * >( data );
   SOFACollidable* sofaCollidable= (SOFACollidable*)params->getCollidableId();
-  ///TODO check destructor, thread safety and so on...
   delete sofaCollidable;
   return PeriodicThread::CALLBACK_DONE;
 }
@@ -417,7 +412,6 @@ PeriodicThread::CallbackCode SOFACallbacks::removeRigidBody( void *data ) {
 
   // Remove body from world and delete
   SOFARigidBody* body = (SOFARigidBody*)params->getBodyId();
-  ///TODO implement m_dynamicsWorld and getRigidBody()
   //sofa_data->m_dynamicsWorld->removeRigidBody( &body->getRigidBody() );
   delete body;
 
@@ -427,8 +421,6 @@ PeriodicThread::CallbackCode SOFACallbacks::removeRigidBody( void *data ) {
 PeriodicThread::CallbackCode SOFACallbacks::removeJoint( void *data ) {
   JointParameters *params = static_cast< JointParameters * >( data ); 
 
-  ///TODO add implementation later...
-
   return PeriodicThread::CALLBACK_DONE;
 }
 
@@ -490,7 +482,6 @@ physicsThread ( bodyParameters.getEngine() ),
   //rigidMass->setMass(inertiaMatrix);
   rigidBody->addObject(rigidMass);
 
-  ///TODO change this bullet stuff
   // make sure the gContactAddedCallback will be called.
   //rigidBody->setCollisionFlags( rigidBody->getCollisionFlags() |
   //  btCollisionObject::CF_CUSTOM_MATERIAL_CALLBACK );
@@ -570,8 +561,6 @@ void SOFARigidBody::setParameters ( PhysicsEngineParameters::RigidBodyParameters
           massDensityModel= bodyParameters.getMassDensityModel();
         }
         if( !massDensityModel ) {
-          ///TODO SU: implement in SOFA?!
-          // TODO: should use inertia and center of mass but for now just use a box
           //btBoxShape box( btVector3( (btScalar)0.1, 
           //  (btScalar)0.1, 
           //  (btScalar)0.1 ) );
@@ -616,7 +605,6 @@ void SOFARigidBody::setParameters ( PhysicsEngineParameters::RigidBodyParameters
      //       Vec3f half_size( 0.1f, 0.1f, 0.1f );
      //       if( bb ) half_size = bullet_data->m_worldScale*bb->size->getValue();
 
-     //       // TODO: take the position of the bounding box into consideration
      //       btBoxShape box( btVector3( half_size.x, half_size.y, half_size.z ) );
      //       btVector3 local_inertia( 0, 0, 0 );
      //       box.calculateLocalInertia( bodyParameters.getMass(), local_inertia);
@@ -642,7 +630,6 @@ void SOFARigidBody::setParameters ( PhysicsEngineParameters::RigidBodyParameters
   if( bodyParameters.haveAutoDisable() || 
     bodyParameters.haveDisableLinearSpeed() || 
     bodyParameters.haveDisableAngularSpeed() ){
-          ///TODO find equivalent in SOFA?!
   //  if( bodyParameters.getAutoDisable() ) {
   //    rigidBody->setSleepingThresholds( bodyParameters.getDisableLinearSpeed(),
   //                                      bodyParameters.getDisableAngularSpeed() );
@@ -690,7 +677,6 @@ void SOFARigidBody::setParameters ( PhysicsEngineParameters::RigidBodyParameters
     //else
     //  rigidDOF->setTranslation(0.0,0.0,0.0);
 
-    ///TODO check this alternative code!
     //Vec3f start_position = bodyParameters.getStartPosition(); 
     //Quaternion start_rotation ( bodyParameters.getStartOrientation() );
 
@@ -699,21 +685,18 @@ void SOFARigidBody::setParameters ( PhysicsEngineParameters::RigidBodyParameters
     //  sofa::defaulttype::Quat( start_rotation.v.x, start_rotation.v.y, start_rotation.v.z, start_rotation.w ) );    
 
 
-    ///TODO find equivalent in SOFA?!
     //rigidBody->setCenterOfMassTransform( t );
   }
 
   // startLinearVelocity 
   if( bodyParameters.haveStartLinearVelocity() ) {
     const Vec3f &v = bodyParameters.getStartLinearVelocity();
-    ///TODO find equivalent in SOFA?!
     //rigidBody->setLinearVelocity( btVector3( v.x, v.y, v.z ) );
   }
 
   //  startAngularVelocity
   if( bodyParameters.haveStartAngularVelocity() ) {
     const Vec3f &v = bodyParameters.getStartAngularVelocity();
-    ///TODO find equivilant in SOFA?!
     //rigidBody->setAngularVelocity( btVector3( v.x, v.y, v.z ) );
   }
 
@@ -765,7 +748,6 @@ void SOFARigidBody::updateCollidables()
   } 
   else 
   {
-    ///TODO change that Bullet code to SOFA
     //// More than one collision shape required, use composite
     //btCompoundShape* compoundShape= new btCompoundShape ();
 
@@ -795,7 +777,6 @@ void SOFARigidBody::collisionShapeChanged ()
   SOFACallbacks::SOFASpecificData *sofa_data = 
     static_cast< SOFACallbacks::SOFASpecificData * >(physicsThread->getEngineSpecificData());
 
-  ///TODO adapt this code to SOFA
   //// Make sure that the rigid body is not already in the world
   //const btCollisionObjectArray &objects = bullet_data->m_dynamicsWorld->getCollisionObjectArray ();
   //if( objects.findLinearSearch(rigidBody.get())  != objects.size()) {
@@ -862,17 +843,14 @@ void SOFACollidable::setParameters ( PhysicsEngineParameters::CollidableParamete
 SOFACollidableOffset::SOFACollidableOffset( PhysicsEngineParameters::OffsetParameters& offsetParameters )
   : SOFACollidable ( offsetParameters )
 {
-  ///TODO add implementation
 }
 
 void SOFACollidableOffset::addBody( SOFARigidBody& body )
 {
-  ///TODO add implementation
 }
 
 void SOFACollidableOffset::removeBody( SOFARigidBody& body )
 {
-  ///TODO add implementation
 }
 
 SOFACollidableShape::SOFACollidableShape( PhysicsEngineParameters::ShapeParameters& shapeParameters )
@@ -880,7 +858,8 @@ SOFACollidableShape::SOFACollidableShape( PhysicsEngineParameters::ShapeParamete
 {
 
   // Create geometry
-  collidableShape = createCollisionShape ( shapeParameters.getShape() );
+  if( shapeParameters.haveShape() )
+    collidableShape = createCollisionShape ( shapeParameters.getShape() );
 }
 
 // Return an appropriate sofa::core::collisionModel to match the specified X3DGeometryNode
@@ -945,7 +924,6 @@ sofa::core::CollisionModel * SOFACollidableShape::createCollisionShape ( X3DGeom
     H3DFloat radius = sph->radius->getValue();
 
     sofa::component::collision::SphereModel::SPtr sphere = sofa::core::objectmodel::New<sofa::component::collision::SphereModel>();
-    //todo
 
     return NULL;
 
@@ -1004,9 +982,6 @@ sofa::core::CollisionModel * SOFACollidableShape::createCollisionShape ( X3DGeom
     //mesh->preallocateVertices( triangles.size() * 3 );
     for( unsigned int i = 0; i < triangles.size(); ++i ) {
 
-      // TODO: maybe add a SOFA options node or something to e.g.
-      // decide if common edges should be detected here(another
-      // argument to addTriangle
       //mesh->addTriangle( btVector3( (btScalar)triangles[i].a.x,
       //                              (btScalar)triangles[i].a.y,
       //                              (btScalar)triangles[i].a.z ),
