@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -17,7 +17,7 @@
 //    along with H3D API; if not, write to the Free Software
 //    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
-//    A commercial license is also available. Please contact us at 
+//    A commercial license is also available. Please contact us at
 //    www.sensegraphics.com for more information.
 //
 //
@@ -34,8 +34,7 @@
 
 #include <H3D/H3DHapticsDevice.h>
 
-using namespace H3D;  
-using namespace HAPI;
+using namespace H3D;
 using namespace PhysicsEngineParameters;
 
 void SoftBodyFloatAttributeParameters::copyOutputParameters( H3DSoftBodyOutputParameters& src ) {
@@ -56,6 +55,9 @@ H3DSoftBodyNodeParameters::H3DSoftBodyNodeParameters()
 : update_bit_mask ( 0 ),
 all_output( 0 ),
 engine ( NULL ),
+#ifdef H3DPHYSICS_ENGINE_PARAMETERS_INCLUDE_NAME
+name(""),
+#endif
 bodyId ( 0 ),
 physicsMaterial( NULL ),
 deformationStrategy( NULL ),
@@ -63,6 +65,9 @@ geometry ( NULL )
 {
 }
 
+H3DSoftBodyNodeParameters::~H3DSoftBodyNodeParameters() {
+}
+
 void H3DSoftBodyNodeParameters::onGetParametersPhysicsThread () {
   updateOutputs ();
 }
@@ -87,6 +92,7 @@ void H3DSoftBodyNodeParameters::updateOutputsFloat () {
       case SoftBodyFloatAttributeParameters::OUTPUT_EXTERNAL_FORCE_MAGNITUDE:
         updateOutputExternalForceMagnitude ( *output );
         break;
+      default: {}
     }
   }
 }
@@ -106,6 +112,7 @@ void H3DSoftBodyNodeParameters::updateOutputsVec3f () {
       case SoftBodyVec3fAttributeParameters::OUTPUT_EXTERNAL_FORCE:
         updateOutputExternalForce ( *output );
         break;
+      default: {}
     }
   }
 }
@@ -116,6 +123,7 @@ void H3DSoftBodyNodeParameters::updateOutputManipulationForce ( SoftBodyVec3fAtt
       setOutputAttribute ( output, GetForce(), manipulationForceNodeAttributes, manipulationForces );
       break;
     }
+    default: {}
   }
 }
 
@@ -125,15 +133,17 @@ void H3DSoftBodyNodeParameters::updateOutputExternalForce ( SoftBodyVec3fAttribu
       setOutputAttribute ( output, GetForce(), externalForceNodeAttributes, vertexForces );
       break;
     }
+    default: {}
   }
 }
 
 void H3DSoftBodyNodeParameters::updateOutputManipulationForceMagnitude ( SoftBodyFloatAttributeParameters& output ) {
   switch ( output.getUnitType() ) {
-    case SoftBodyFloatAttributeParameters::UNIT_NODE: { 
+    case SoftBodyFloatAttributeParameters::UNIT_NODE: {
       setOutputAttribute ( output, GetForceMag(), manipulationForceMagNodeAttributes, manipulationForces );
       break;
     }
+    default: {}
   }
 }
 
@@ -143,6 +153,7 @@ void H3DSoftBodyNodeParameters::updateOutputExternalForceMagnitude ( SoftBodyFlo
       setOutputAttribute ( output, GetForceMag(), externalForceMagNodeAttributes, vertexForces );
       break;
     }
+    default: {}
   }
 }
 
@@ -160,6 +171,10 @@ void H3DSoftBodyNodeParameters::copyOutputParameters( H3DSoftBodyNodeParameters&
 void H3DSoftBodyNodeParameters::copyInputParameters( H3DSoftBodyNodeParameters& src ) {
   copyOutputFlags ( src.update_bit_mask );
 
+#ifdef H3DPHYSICS_ENGINE_PARAMETERS_INCLUDE_NAME
+  name = src.name;
+#endif
+
   if ( src.haveTransform() ) {
     transform= src.transform;
   }
@@ -170,7 +185,7 @@ void H3DSoftBodyNodeParameters::copyInputParameters( H3DSoftBodyNodeParameters&
     // sub-classes of H3DPhysicsMaterialNodes with different
     // H3DPhysicsMaterialPropertyNodes w.o memory leakage.
     bool changeAll = false;
-    if( havePhysicsMaterial() && 
+    if( havePhysicsMaterial() &&
       physicsMaterial != src.physicsMaterial )
       changeAll = true;
 
@@ -189,13 +204,13 @@ void H3DSoftBodyNodeParameters::copyInputParameters( H3DSoftBodyNodeParameters&
     if ( src.haveDeformationStrategyParameters() ) {
 
       if( haveDeformationStrategyParameters() )
-        deformationStrategyParameters.get()->copyInputParameters( *src.getDeformationStrategyParameters() );        
+        deformationStrategyParameters.get()->copyInputParameters( *src.getDeformationStrategyParameters() );
       else
         setDeformationStrategyParameters( src.getDeformationStrategyParameters() );
     }
   }
   if ( src.haveGeometry() ) {
-    geometry= src.geometry;  
+    geometry= src.geometry;
   }
   if ( src.haveSurfaceGeometry () ) {
     surfaceGeometry= src.surfaceGeometry;
@@ -230,14 +245,14 @@ VertexBodyConstraintParameters::VertexBodyConstraintParameters()
 void VertexBodyConstraintParameters::copyOutputParameters( ConstraintParameters& src ) {
   VertexBodyConstraintParameters *s = dynamic_cast< VertexBodyConstraintParameters * >( &src );
   if( s ) {
-    ConstraintParameters::copyOutputParameters ( *s );    
+    ConstraintParameters::copyOutputParameters ( *s );
   }
 }
 
 void VertexBodyConstraintParameters::copyInputParameters( ConstraintParameters& src ) {
   VertexBodyConstraintParameters *s = dynamic_cast< VertexBodyConstraintParameters * >( &src );
   if( s ) {
-    ConstraintParameters::copyInputParameters ( *s );    
+    ConstraintParameters::copyInputParameters ( *s );
 
     if ( s->haveIndex() ) {
       index= s->index;
@@ -254,7 +269,7 @@ body2 ( 0 )
 void H3DAttachmentParameters::copyOutputParameters( ConstraintParameters& src ) {
   H3DAttachmentParameters *s = dynamic_cast< H3DAttachmentParameters * >( &src );
   if( s ) {
-    ConstraintParameters::copyOutputParameters ( *s );    
+    ConstraintParameters::copyOutputParameters ( *s );
   }
 }
 
@@ -262,7 +277,7 @@ void H3DAttachmentParameters::copyInputParameters( ConstraintParameters& src ) {
 
   H3DAttachmentParameters *s = dynamic_cast< H3DAttachmentParameters * >( &src );
   if( s ) {
-    ConstraintParameters::copyInputParameters ( *s );    
+    ConstraintParameters::copyInputParameters ( *s );
 
     if ( s->haveBody2() ) {
       body2= s->body2;
@@ -284,14 +299,14 @@ H3DSoftBodyAttachmentParameters::H3DSoftBodyAttachmentParameters()
 }
 
 void H3DSoftBodyAttachmentParameters::copyOutputParameters( ConstraintParameters& s ) {
-  H3DSoftBodyAttachmentParameters *src = dynamic_cast< H3DSoftBodyAttachmentParameters* >( &s ); 
+  H3DSoftBodyAttachmentParameters *src = dynamic_cast< H3DSoftBodyAttachmentParameters* >( &s );
   if( src ) {
     H3DAttachmentParameters::copyOutputParameters ( *src );
   }
 }
 
 void H3DSoftBodyAttachmentParameters::copyInputParameters( ConstraintParameters& s ) {
-  H3DSoftBodyAttachmentParameters *src = dynamic_cast< H3DSoftBodyAttachmentParameters * >( &s ); 
+  H3DSoftBodyAttachmentParameters *src = dynamic_cast< H3DSoftBodyAttachmentParameters * >( &s );
   if( src ) {
     H3DAttachmentParameters::copyInputParameters ( *src );
 
@@ -311,7 +326,7 @@ void SoftBodyAttachmentParameters::copyOutputParameters( H3DAttachmentParameters
 }
 
 void SoftBodyAttachmentParameters::copyInputParameters( H3DAttachmentParameters& src ) {
-  SoftBodyAttachmentParameters* params= 
+  SoftBodyAttachmentParameters* params=
     dynamic_cast<SoftBodyAttachmentParameters*>(&src);
   H3DSoftBodyAttachmentParameters::copyInputParameters ( src );
 
@@ -323,7 +338,7 @@ void SoftBodyAttachmentParameters::copyInputParameters( H3DAttachmentParameters&
     // sub-classes of H3DPhysicsMaterialNodes with different
     // H3DPhysicsMaterialPropertyNodes w.o memory leakage.
     bool changeAll = false;
-    if( havePhysicsMaterial() && 
+    if( havePhysicsMaterial() &&
       physicsMaterial != params->physicsMaterial )
       changeAll = true;
 
@@ -346,14 +361,14 @@ SoftBodyLinearJointParameters::SoftBodyLinearJointParameters()
   type = "SoftBodyLinearJoint";
 }
 void SoftBodyLinearJointParameters::copyOutputParameters( ConstraintParameters& s ) {
-  SoftBodyLinearJointParameters *src = dynamic_cast< SoftBodyLinearJointParameters * >( &s ); 
+  SoftBodyLinearJointParameters *src = dynamic_cast< SoftBodyLinearJointParameters * >( &s );
   if( src ) {
     JointParameters::copyOutputParameters ( *src );
   }
 }
 
 void SoftBodyLinearJointParameters::copyInputParameters( ConstraintParameters& s ) {
-  SoftBodyLinearJointParameters *src = dynamic_cast< SoftBodyLinearJointParameters * >( &s ); 
+  SoftBodyLinearJointParameters *src = dynamic_cast< SoftBodyLinearJointParameters * >( &s );
   if( src ) {
     JointParameters::copyInputParameters ( *src );
 
@@ -370,14 +385,14 @@ JointParameters()
 }
 
 void SoftBodyAngularJointParameters::copyOutputParameters( ConstraintParameters& s ) {
-  SoftBodyAngularJointParameters *src = dynamic_cast< SoftBodyAngularJointParameters * >( &s ); 
+  SoftBodyAngularJointParameters *src = dynamic_cast< SoftBodyAngularJointParameters * >( &s );
   if( src ) {
     JointParameters::copyOutputParameters ( *src );
   }
 }
 
 void SoftBodyAngularJointParameters::copyInputParameters( ConstraintParameters& s ) {
-  SoftBodyAngularJointParameters *src = dynamic_cast< SoftBodyAngularJointParameters * >( &s ); 
+  SoftBodyAngularJointParameters *src = dynamic_cast< SoftBodyAngularJointParameters * >( &s );
   if( src ) {
     JointParameters::copyInputParameters ( *src );
 
@@ -397,7 +412,7 @@ void ModifierParameters::copyInputParameters( ModifierParameters& s ) {
   device_index = s.device_index;
 }
 void DeformationStrategyParameters::copyOutputParameters( DeformationStrategyParameters& s ) {
-  copyOutputFlags ( s.update_bit_mask );  
+  copyOutputFlags ( s.update_bit_mask );
 }
 
 void DeformationStrategyParameters::copyInputParameters( DeformationStrategyParameters& s ) {
@@ -408,7 +423,7 @@ void DeformationStrategyParameters::copyInputParameters( DeformationStrategyPara
   }
 }
 void H3DPhysicsMaterialParameters::copyOutputParameters( H3DPhysicsMaterialParameters& s ) {
-  copyOutputFlags ( s.update_bit_mask );  
+  copyOutputFlags ( s.update_bit_mask );
 }
 
 void H3DPhysicsMaterialParameters::copyInputParameters( H3DPhysicsMaterialParameters& s ) {
@@ -416,9 +431,9 @@ void H3DPhysicsMaterialParameters::copyInputParameters( H3DPhysicsMaterialParame
 
   if ( s.haveDamping() ) {
     // If the damping field is changed to another node instead of copying
-    // changed parameters, change the pointer completetely. 
+    // changed parameters, change the pointer completetely.
     bool changeAll = false;
-    if( haveDamping() && 
+    if( haveDamping() &&
       damping != s.damping )
       changeAll = true;
 
@@ -433,9 +448,9 @@ void H3DPhysicsMaterialParameters::copyInputParameters( H3DPhysicsMaterialParame
 
   if ( s.haveMass() ) {
     // If the mass field is changed to another node instead of copying
-    // changed parameters, change the pointer completetely. 
+    // changed parameters, change the pointer completetely.
     bool changeAll = false;
-    if( haveMass() && 
+    if( haveMass() &&
       mass != s.mass )
       changeAll = true;
 
@@ -450,9 +465,9 @@ void H3DPhysicsMaterialParameters::copyInputParameters( H3DPhysicsMaterialParame
 
   if ( s.haveFriction() ) {
     // If the friction field is changed to another node instead of copying
-    // changed parameters, change the pointer completetely. 
+    // changed parameters, change the pointer completetely.
     bool changeAll = false;
-    if( haveFriction() && 
+    if( haveFriction() &&
       friction != s.friction )
       changeAll = true;
 
@@ -474,21 +489,21 @@ H3DPhysicsMaterialParameters()
 }
 
 void PhysicsMaterialParameters::copyOutputParameters( H3DPhysicsMaterialParameters& s ) {
-  copyOutputFlags ( s.update_bit_mask );  
+  copyOutputFlags ( s.update_bit_mask );
 }
 
 void PhysicsMaterialParameters::copyInputParameters( H3DPhysicsMaterialParameters& s ) {
   copyOutputFlags ( s.update_bit_mask );
 
-  PhysicsMaterialParameters *src = dynamic_cast< PhysicsMaterialParameters * >( &s ); 
+  PhysicsMaterialParameters *src = dynamic_cast< PhysicsMaterialParameters * >( &s );
   if( src ) {
     H3DPhysicsMaterialParameters::copyInputParameters ( *src );
 
     if ( src->haveElasticity() ) {
       // If the elasticity field is changed to another node instead of copying
-      // changed parameters, change the pointer completetely. 
+      // changed parameters, change the pointer completetely.
       bool changeAll = false;
-      if( haveElasticity() && 
+      if( haveElasticity() &&
         elasticity != src->elasticity )
         changeAll = true;
 
@@ -508,21 +523,21 @@ MassSpringPhysicsMaterialParameters::MassSpringPhysicsMaterialParameters()
 }
 
 void MassSpringPhysicsMaterialParameters::copyOutputParameters( H3DPhysicsMaterialParameters& s ) {
-  copyOutputFlags ( s.update_bit_mask );  
+  copyOutputFlags ( s.update_bit_mask );
 }
 
 void MassSpringPhysicsMaterialParameters::copyInputParameters( H3DPhysicsMaterialParameters& s ) {
   copyOutputFlags ( s.update_bit_mask );
 
-  MassSpringPhysicsMaterialParameters *src = dynamic_cast< MassSpringPhysicsMaterialParameters * >( &s ); 
+  MassSpringPhysicsMaterialParameters *src = dynamic_cast< MassSpringPhysicsMaterialParameters * >( &s );
   if( src ) {
     H3DPhysicsMaterialParameters::copyInputParameters ( *src );
 
     if ( src->haveStiffness() ) {
       // If the stiffness field is changed to another node instead of copying
-      // changed parameters, change the pointer completetely. 
+      // changed parameters, change the pointer completetely.
       bool changeAll = false;
-      if( haveStiffness() && 
+      if( haveStiffness() &&
         stiffness != src->stiffness )
         changeAll = true;
 
@@ -546,21 +561,21 @@ PhysicsMaterialParameters()
 }
 
 void FEMPhysicsMaterialParameters::copyOutputParameters( H3DPhysicsMaterialParameters& s ) {
-  copyOutputFlags ( s.update_bit_mask );  
+  copyOutputFlags ( s.update_bit_mask );
 }
 
 void FEMPhysicsMaterialParameters::copyInputParameters( H3DPhysicsMaterialParameters& s ) {
   copyOutputFlags ( s.update_bit_mask );
 
-  FEMPhysicsMaterialParameters *src = dynamic_cast< FEMPhysicsMaterialParameters * >( &s ); 
+  FEMPhysicsMaterialParameters *src = dynamic_cast< FEMPhysicsMaterialParameters * >( &s );
   if( src ) {
     PhysicsMaterialParameters::copyInputParameters ( *src );
 
     if ( src->havePoissonRatio() ) {
       // If the poissonRatio field is changed to another node instead of copying
-      // changed parameters, change the pointer completetely. 
+      // changed parameters, change the pointer completetely.
       bool changeAll = false;
-      if( havePoissonRatio() && 
+      if( havePoissonRatio() &&
         poissonRatio != src->poissonRatio )
         changeAll = true;
 
@@ -577,7 +592,7 @@ void FEMPhysicsMaterialParameters::copyInputParameters( H3DPhysicsMaterialParame
 }
 
 void MaterialPropertyParameters::copyOutputParameters( MaterialPropertyParameters& s ) {
-  copyOutputFlags ( s.update_bit_mask );  
+  copyOutputFlags ( s.update_bit_mask );
 }
 
 void MaterialPropertyParameters::copyInputParameters( MaterialPropertyParameters& s ) {
@@ -590,10 +605,10 @@ void MaterialPropertyParameters::copyInputParameters( MaterialPropertyParameters
 void FloatMaterialPropertyParameters::copyInputParameters( MaterialPropertyParameters& s ) {
   MaterialPropertyParameters::copyInputParameters ( s );
 
-  FloatMaterialPropertyParameters *src = dynamic_cast< FloatMaterialPropertyParameters * >( &s ); 
+  FloatMaterialPropertyParameters *src = dynamic_cast< FloatMaterialPropertyParameters * >( &s );
   if( src ) {
     if ( src->haveValue() ) {
       value = src->value;
-    } 
+    }
   }
 }
